/****************************************************************************/
/* DIET forwarder implementation - MA DAG forwarder implementation          */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Gael Le Mahec   (gael.le.mahec@ens-lyon.fr)                         */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.4  2010/07/27 16:16:49  glemahec
 * Forwarders robustness
 *
 * Revision 1.3  2010/07/14 23:45:30  bdepardo
 * Header corrections
 *
 * Revision 1.2  2010/07/13 15:24:13  glemahec
 * Warnings corrections and some robustness improvements
 *
 * Revision 1.1  2010/07/12 16:08:56  glemahec
 * DIET 2.5 beta 1 - Forwarder implementations
 ****************************************************************************/

#include "DIETForwarder.hh"
#include "ORBMgr.hh"
#include <string>

using namespace std;

::CORBA::Long DIETForwarder::processDagWf(const ::corba_wf_desc_t& dag_desc,
																					const char* cltMgrRef,
																					::CORBA::Long wfReqId,
																					const char* objName)
{
	string objString(objName);
	string name;
	
	if (!remoteCall(objString)) {
		return getPeer()->processDagWf(dag_desc, cltMgrRef, wfReqId, objString.c_str());
	}
	
	name = getName(objString);
		
	MaDag_var agent = ORBMgr::getMgr()->resolve<MaDag, MaDag_var>(MADAGCTXT, name,
																																this->name);
	return agent->processDagWf(dag_desc, cltMgrRef, wfReqId);
}

::CORBA::Long DIETForwarder::processMultiDagWf(const ::corba_wf_desc_t& dag_desc,
																							 const char* cltMgrRef,
																							 ::CORBA::Long wfReqId,
																							 ::CORBA::Boolean release,
																							 const char* objName)
{
	string objString(objName);
	string name;
	
	if (!remoteCall(objString)) {
		return getPeer()->processMultiDagWf(dag_desc, cltMgrRef, wfReqId, release,
																	 objString.c_str());
	}
	
	name = getName(objString);
		
	MaDag_var agent = ORBMgr::getMgr()->resolve<MaDag, MaDag_var>(MADAGCTXT, name,
																																this->name);
	return agent->processMultiDagWf(dag_desc, cltMgrRef, wfReqId, release);	
}

::CORBA::Long DIETForwarder::getWfReqId(const char* objName) {
	string objString(objName);
	string name;
	
	if (!remoteCall(objString)) {
		return getPeer()->getWfReqId(objString.c_str());
	}
	
	name = getName(objString);
	
	MaDag_var agent = ORBMgr::getMgr()->resolve<MaDag, MaDag_var>(MADAGCTXT, name,
																																this->name);
	return agent->getWfReqId();	
}

void DIETForwarder::releaseMultiDag(::CORBA::Long wfReqId,
																		const char* objName)
{
	string objString(objName);
	string name;
	
	if (!remoteCall(objString)) {
		return getPeer()->releaseMultiDag(wfReqId, objString.c_str());
	}
	
	name = getName(objString);
		
	MaDag_var agent = ORBMgr::getMgr()->resolve<MaDag, MaDag_var>(MADAGCTXT, name,
																																this->name);
	return agent->releaseMultiDag(wfReqId);	
}

void DIETForwarder::cancelDag(::CORBA::Long dagId,
															const char* objName)
{
	string objString(objName);
	string name;
	
	if (!remoteCall(objString)) {
		return getPeer()->cancelDag(dagId, objString.c_str());
	}
	
	name = getName(objString);
		
	MaDag_var agent = ORBMgr::getMgr()->resolve<MaDag, MaDag_var>(MADAGCTXT, name,
																																this->name);
	return agent->cancelDag(dagId);	
}

void DIETForwarder::setPlatformType(::MaDag::pfmType_t pfmType,
																		const char* objName)
{
	string objString(objName);
	string name;
	
	if (!remoteCall(objString)) {
		return getPeer()->setPlatformType(pfmType, objString.c_str());
	}
	
	name = getName(objString);
		
	MaDag_var agent = ORBMgr::getMgr()->resolve<MaDag, MaDag_var>(MADAGCTXT, name,
																																this->name);
	return agent->setPlatformType(pfmType);	
}
