/****************************************************************************/
/* DIET forwarder implementation - SeD forwarder implementation             */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Gael Le Mahec   (gael.le.mahec@ens-lyon.fr)                         */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.5  2011/05/09 13:10:10  bdepardo
 * Added method diet_get_SeD_services to retreive the services of a SeD given
 * its name
 *
 * Revision 1.4  2010/07/27 16:16:49  glemahec
 * Forwarders robustness
 *
 * Revision 1.3  2010/07/14 23:45:30  bdepardo
 * Header corrections
 *
 * Revision 1.2  2010/07/13 15:24:13  glemahec
 * Warnings corrections and some robustness improvements
 *
 * Revision 1.1  2010/07/12 16:08:56  glemahec
 * DIET 2.5 beta 1 - Forwarder implementations
 ****************************************************************************/

#include "DIETForwarder.hh"
#include "ORBMgr.hh"
#include <string>

using namespace std;

::CORBA::Long DIETForwarder::checkContract(::corba_estimation_t& estimation,
                                           const ::corba_pb_desc_t& pb,
                                           const char* objName)
{
  string objString(objName);
  string name;

  if (!remoteCall(objString)) {
    return getPeer()->checkContract(estimation, pb, objString.c_str());
  }
	
  name = getName(objString);
	
  SeD_var sed = ORBMgr::getMgr()->resolve<SeD, SeD_var>(SEDCTXT, name,
                                                        this->name);
  return sed->checkContract(estimation, pb);
}

void DIETForwarder::updateTimeSinceLastSolve(const char* objName) {
  string objString(objName);
  string name;
	
  if (!remoteCall(objString)) {
    return getPeer()->updateTimeSinceLastSolve(objString.c_str());
  }
	
  name = getName(objString);

  SeD_var sed = ORBMgr::getMgr()->resolve<SeD, SeD_var>(SEDCTXT, name,
                                                        this->name);
  return sed->updateTimeSinceLastSolve();
	
}

::CORBA::Long DIETForwarder::solve(const char* path,
                                   ::corba_profile_t& pb,
                                   const char* objName)
{
  string objString(objName);
  string name;
	
  if (!remoteCall(objString)) {
    return getPeer()->solve(path, pb, objString.c_str());
  }
	
  name = getName(objString);

  SeD_var sed = ORBMgr::getMgr()->resolve<SeD, SeD_var>(SEDCTXT, name,
                                                        this->name);
  return sed->solve(path, pb);
}

void DIETForwarder::solveAsync(const char* path,
                               const ::corba_profile_t& pb,
                               const char* volatileclientPtr,
                               const char* objName)
{
  string objString(objName);
  string name;
	
  if (!remoteCall(objString)) {
    return getPeer()->solveAsync(path, pb, volatileclientPtr, objString.c_str());
  }
	
  name = getName(objString);

  SeD_var sed = ORBMgr::getMgr()->resolve<SeD, SeD_var>(SEDCTXT, name,
                                                        this->name);
  return sed->solveAsync(path, pb, volatileclientPtr);
}

#ifdef HAVE_DAGDA
char* DIETForwarder::getDataMgrID(const char* objName) {
  string objString(objName);
  string name;
	
  if (!remoteCall(objString)) {
    return getPeer()->getDataMgrID(objString.c_str());
  }
	
  name = getName(objString);

  SeD_var sed = ORBMgr::getMgr()->resolve<SeD, SeD_var>(SEDCTXT, name,
                                                        this->name);
  return sed->getDataMgrID();
}
#endif

SeqCorbaProfileDesc_t* 
DIETForwarder::getSeDProfiles(::CORBA::Long& length,
                              const char* objName) {
  string objString(objName);
  string name;
  
  if (!remoteCall(objString)) {
    return getPeer()->getSeDProfiles(length, objString.c_str());
  }
	
  name = getName(objString);
	
  SeD_var sed = ORBMgr::getMgr()->resolve<SeD, SeD_var>(SEDCTXT,
                                                        name,
                                                        this->name);
  return sed->getSeDProfiles(length);
}
