/****************************************************************************/
/* DIET Data Manager implementation header                                  */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Bruno DEL FABBRO (Bruno.DelFabbro@lifc.univ-fcomte)                 */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.17  2010/12/17 09:47:59  kcoulomb
 * * Set diet to use the new log with forwarders
 * * Fix a CoRI problem
 * * Add library version remove DTM flag from ccmake because deprecated
 *
 * Revision 1.16  2010/07/12 16:14:10  glemahec
 * DIET 2.5 beta 1 - Use the new ORB manager and allow the use of SSH-forwarders for all DIET CORBA objects
 *
 * Revision 1.15  2008/07/16 00:46:48  ecaron
 * Remove HAVE_ALTPREDICT (deprecated code)
 *
 * Revision 1.14  2005/09/05 16:01:34  hdail
 * Addition of locationID information and getDataLoc method call.
 *
 * Revision 1.13  2005/04/08 13:02:43  hdail
 * The code for LogCentral has proven itself stable and it seems bug free.
 * Since no external libraries are required to compile in LogCentral, its now
 * going to be compiled in by default always ... its usage is easily controlled by
 * configuration file.
 *
 * Revision 1.12  2004/12/06 07:32:50  bdelfabr
 * cleanup memory management for tranfers between servers.
 *
 * Revision 1.11  2004/10/06 11:59:04  bdelfabr
 * corrected inout bug (I hope so)
 *
 * Revision 1.10  2004/10/05 08:23:09  bdelfabr
 * fixing bug for persistent file : add a changePath method thta gives the good file access path
 *
 * Revision 1.9  2004/03/01 18:42:08  rbolze
 * add logservice
 *
 * Revision 1.8  2004/02/27 10:26:37  bdelfabr
 * let DIET_PERSISTENCE_MODE set to 1, coding standard
 *
 * Revision 1.7  2003/12/01 14:49:30  pcombes
 * Rename dietTypes.hh to DIET_data_internal.hh, for more coherency.
 *
 * Revision 1.6  2003/11/10 14:03:10  bdelfabr
 * adding methods that allow tranfer time between servers to be computed
 *
 * Revision 1.5  2003/10/14 20:29:06  bdelfabr
 * adding print methods (PERSISTENT mode only)
 *
 * Revision 1.4  2003/10/01 07:06:42  bdelfabr
 * removing DataManagerID.hh which is not used
 *
 * Revision 1.3  2003/09/30 15:08:57  bdelfabr
 * dlist are replaced by stl map?
 *
 * Revision 1.2  2003/09/24 09:16:18  pcombes
 * Merge corba_DataMgr_desc_t and corba_data_desc_t.
 * DataMgr does not need a name: use its reference.
 *
 * Revision 1.1  2003/09/22 21:07:21  pcombes
 * Set all the modules and their interfaces for data persistency.
 ***************************************************************************/

#ifndef _DATAMGRIMPL_HH_
#define _DATAMGRIMPL_HH_

#include <map>


#include "DataMgr.hh"
#include "ChildID.hh"
#include "common_types.hh"
#include "DIET_data_internal.hh"
#include "LocMgr.hh"
#include "ServiceTable.hh"
#include "LinkedList.hh"
#include "ts_container/ts_map.hh"
#ifdef USE_LOG_SERVICE
#include "DietLogComponent.hh"
#endif
#include "Forwarder.hh"
#include "DataMgrFwdr.hh"

/** defines string comparison for map management */

struct cmpID
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }
};


class DataMgrImpl : public POA_DataMgr,
		    public PortableServer::RefCountServantBase
{
public:                                              

  DataMgrImpl(const char* id);

  ~DataMgrImpl();

  int
  run();

#ifdef USE_LOG_SERVICE
  /**
   * Sets the dietLogComponent of this DataMgr. If this function
   * is not called or the parameter is NULL, no monitoring information
   * will be gathered.
   */
  void
  setDietLogComponent(DietLogComponent* dietLogComponent);
#endif
  void
  printvalue(double *value,long unsigned int size);
  /** look for data presence */

  bool
  dataLookup(char* argID);

  /** update add and remove data */
  void
  updateDataRefOrder(corba_data_t& arg);

  /** Copy data from list to SeD structure */
  void
  getData(corba_data_t& arg);

  /** addData to the dataDescLsit */
  void
  addData(corba_data_t& arg, int inout);

  /** invoke remote DataManger, it has to send the data */
  virtual void
  putData(const char* argID, const char* me);

  /** remove data from the list */
  virtual CORBA::Long
  rmDataRefDataMgr(const char* argID);

  /** looking for dataManager localization */
  virtual char* whereData(const char* argID);

  /** send data to a DataManager */
  virtual void
  sendData(corba_data_t& arg);

  void 
  changePath(corba_data_t &arg, char * newPath);
 
  virtual char *
  setMyName();

  /* print list of data owned */
  void 
  printList();
  
  void
  printList1();

  void
  updateDataProperty(corba_data_t& arg);

  void
  updateDataList(corba_data_t& arg);

#if 0
  void
  rmAllData();
#endif

  /** looking for dataManager owner */ 
  virtual char *
  whichSeDOwner(const char * argId);

  /** looking for dataManager owner */ 
  char *
  whichDataMgr(const char * argId);

private:
 
  /**************************************************************************/
  /* Private fields                                                         */
  /**************************************************************************/
  char* id;
  /** Local host name */
  char localHostName[256];
  /** ID of this DataMgr amongst the children of its parent */
  ChildID childID;
  /* reference of the parent Data Location Manager */
  LocMgr_var parent;
  /** defines the type List of data */
  typedef ts_map<const char*,corba_data_t,cmpID> dietDataDescList_t;
  /** mutex list */
  typedef ts_map<const char *,omni_mutex,cmpID> dietDataIDLockList_t;
  
  /** List of the data held by the Data Manager*/
  dietDataDescList_t dataDescList;
  
  /** List of lock to manage data transmission */
  dietDataIDLockList_t lockList;

#ifdef USE_LOG_SERVICE
  /**
   * The dietLogComponent. This ptr can be null to indicate that
   * no monitoring information must be generated, so don't forget
   * to check it before usage.
   */
  DietLogComponent* dietLogComponent;
#endif
  /**************************************************************************/
  /* Private methods                                                        */
  /**************************************************************************/
  /** copy corba_data_desc_t in corba_data_t */
  corba_data_t
  moveToCorbaDataT(corba_data_desc_t& dataDesc);
  /** copy a list element to a corba_data_t argument */
  void
  cpEltListToDataT(corba_data_t *arg);
  /** mutex management */
  bool
  isInLockList(char* argID);
  void
  dataIDUnlock(const corba_data_id_t& dataId);
  void
  dataIDLock(const corba_data_id_t& cdataId);
  /** remove data from list */
  void
  rmDataDescFromList(char* argID);
  /** add data in the list */
  void
  addDataDescToList(corba_data_t* arg, int inout);
  /** mutex management */
  void
  addDataIDToLockList(char* argID);
  void
  rmDataFromIDList(char* argID);
  /** copy a corba_data_t to a corba_data_desc_t argument */
  corba_data_desc_t
  moveToCorbaDataDesc(corba_data_t& arg);
  void 
  persistent_data_release(corba_data_t* arg);

};

class DataMgrFwdrImpl : public POA_DataMgr,
	public PortableServer::RefCountServantBase
{
private:
	Forwarder_ptr forwarder;
	char* objName;
public:
	DataMgrFwdrImpl(Forwarder_ptr fwdr, const char* objName);
	virtual void putData(const char* argID, const char* me);
	virtual CORBA::Long rmDataRefDataMgr(const char* argID);
	virtual char* whereData(const char* argID);
	virtual void sendData(corba_data_t& arg);
	virtual void printList();
	virtual char* setMyName();
	virtual char* whichSeDOwner(const char * argId);
};
#endif // _DATAMGRIMPL_HH_
