/****************************************************************************/
/* DIET agent globlal schedulers implementation : add yours !!!             */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Philippe COMBES (Philippe.Combes@ens-lyon.fr)                       */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.27  2011/04/05 14:03:10  bdepardo
 * Replaced cout by WARNING
 *
 * Revision 1.26  2011/02/24 11:55:46  bdepardo
 * Use the new CONFIG_XXX macros.
 *
 * Revision 1.25  2011/02/09 15:09:55  hguemar
 * configuration backend changed again: more CONFIG_XXX
 *
 * Revision 1.24  2011/02/04 15:29:54  hguemar
 * fix some GCC warnings
 *
 * Revision 1.23  2011/02/04 15:20:48  hguemar
 * fixes to new configuration parser
 * some cleaning
 *
 * Revision 1.22  2010/03/04 08:56:09  bdepardo
 * Include C++ headers instead of C headers.
 * strchr returns a const char* and not a char*, hence, in order to compile
 * with gcc >= 4.4.1 we need to retrieve the result of strchr into a
 * const char* and not just a char*.
 *
 * Revision 1.21  2010/03/03 10:19:03  bdepardo
 * Changed \n into endl
 *
 * Revision 1.20  2009/09/24 10:48:34  bdepardo
 * Fixed a bug when reverting to default scheduler: StdGS wasn't initialized
 * (in deserialize).
 *
 * Revision 1.19  2009/09/23 15:24:03  bdepardo
 * Fixed a bug when reverting to default scheduler: StdGS wasn't initialized.
 *
 * Revision 1.18  2008/01/01 19:04:46  ycaniou
 * Only cosmetic
 *
 * Revision 1.17  2007/11/29 13:04:19  glemahec
 * Bug correction: with gcc >=4.1 cross compilation error when creating an object
 * in a case statement which is not the last one. The error appears only with
 * USERSCHED option activated.
 *
 * Revision 1.16  2007/03/27 08:49:27  glemahec
 * deserialize and chooseGlobalScheduler methods now return an instance of a dynamically loaded class if the aggregator is DIET_AGG_USER.
 *
 * Revision 1.15  2006/11/16 09:55:54  eboix
 *   DIET_config.h is no longer used. --- Injay2461
 *
 * Revision 1.14  2006/05/15 20:15:28  ecaron
 * Correct strtok_r bug on MacOSX for stdGS::deserialize (using strsep)
 *
 * Revision 1.13  2006/05/15 19:56:07  ecaron
 * Correct the bug of wrong usage of strtok_r on MacOSX
 * Update with strsep function (Thanks Alan and Pilou)
 *
 * Revision 1.12  2006/05/15 19:37:42  ecaron
 * *** empty log message ***
 *
 * Revision 1.11  2006/01/19 21:35:42  pfrauenk
 * CoRI : when --enable-cori - round-robin is the default scheduler -
 *        CoRI is not called (any more) for collecting information
 *        (so no FAST possible any more)
 *
 * Revision 1.10  2005/08/31 14:44:41  alsu
 * New plugin scheduling interface: minor changes
 *
 * Revision 1.9  2005/05/16 15:22:52  alsu
 * mostly small stuff, one big (stupid) bug
 *
 * Revision 1.8  2005/05/16 12:27:24  alsu
 * removing hard-coded nameLength fields
 *
 * Revision 1.7  2005/05/15 15:47:04  alsu
 * - implementing PriorityScheduler
 * - minor change to the chooseGlobalScheduler method
 *
 * Revision 1.6  2004/12/08 15:02:51  alsu
 * plugin scheduler first-pass validation testing complete.  merging into
 * main CVS trunk; ready for more rigorous testing.
 *
 * Revision 1.5.2.4  2004/12/06 16:05:57  alsu
 * using RR scheduler only after the generic MIN/MAX schedulers
 *
 * Revision 1.5.2.3  2004/11/06 16:23:19  alsu
 * reordering adding RR, generic min, and generic max aggregators
 *
 * Revision 1.5.2.2  2004/10/31 22:17:30  alsu
 * minor code cleanup and adding the RR aggregator to the "scheduler"
 * stack.
 *
 * Revision 1.5.2.1  2004/10/27 22:35:50  alsu
 * include
 *
 * Revision 1.5  2004/10/15 08:21:17  hdail
 * - Removed references to corba_response_t->sortedIndexes - no longer useful.
 * - Removed sort functions -- they have been replaced by aggregate and are never
 *   called.
 *
 * Revision 1.4  2004/09/14 12:43:55  hdail
 * Changed cleanup of ser_sched from delete to free to agree with alloc.
 *
 * Revision 1.3  2003/07/04 09:47:59  pcombes
 * Use new ERROR, WARNING and TRACE macros.
 *
 * Revision 1.2  2003/05/05 14:29:51  pcombes
 * Add "all-steps" traces in all methods.
 *
 * Revision 1.1  2003/04/10 12:58:08  pcombes
 * Interface for global schedulers, called by agents and associated to
 * requests. Add an implementation of this interface: StdGS.
 ****************************************************************************/
#include "GlobalSchedulers.hh"
#include <iostream>
using namespace std;
#include <cstdio>
#include <cstring>

#include "debug.hh"
#include "Vector.h"

/* New : For scheduler load support. */
#ifdef USERSCHED
#include "UserScheduler.hh"
#include "configuration.hh"
#endif

/** The trace level. */
extern unsigned int TRACE_LEVEL;

// Use SCHED_CLASS for the name of the class
// (this->name cannot be used in static member functions)
#define SCHED_TRACE_FUNCTION(formatted_text)      \
  TRACE_TEXT(TRACE_ALL_STEPS, SCHED_CLASS << "::");\
  TRACE_FUNCTION(TRACE_ALL_STEPS,formatted_text)


/****************************************************************************/
/* GlobalScheduler: parent class                                            */
/****************************************************************************/
#undef SCHED_CLASS
#define SCHED_CLASS "GlobalScheduler"

GlobalScheduler::GlobalScheduler()
{
   this->name = "GlobalScheduler";
}

/** Destroy the schedulers list */
GlobalScheduler::~GlobalScheduler()
{
   this->schedulers.emptyIt();
}


/**
 * Return the GlobalScheduler deserialized from the string
 * \c serializedScheduler.
 */
GlobalScheduler*
GlobalScheduler::deserialize(const char* serializedScheduler)
{
  SCHED_TRACE_FUNCTION(serializedScheduler);
  int nameLength;

  // TODO: is the isolated scope required ?
  {
    const char *colon;
    if ((colon = strchr(serializedScheduler, ':'))) {
      nameLength = colon - serializedScheduler;
    }
    else {
      nameLength = strlen(serializedScheduler);
    }
  }

  if (!strncmp(serializedScheduler, StdGS::stName, nameLength)) {
    return StdGS::deserialize(serializedScheduler);
  }
  else if (!strncmp(serializedScheduler, PriorityGS::stName, nameLength)) {
    return PriorityGS::deserialize(serializedScheduler);
  }
  else
    /* New : For scheduler load support. */
#ifdef USERSCHED
    if (!strncmp(serializedScheduler, UserScheduler::stName,nameLength)) {
      std::string moduleName;
      if (!CONFIG_STRING(diet::MODULENAME, moduleName)) {
        WARNING("moduleName parameter is not set in the configuration file ; "
                << "reverting to default (StdGS)" << std::endl);
        return (GlobalScheduler::chooseGlobalScheduler());
      }
      try {
        return UserScheduler::deserialize(serializedScheduler,
                                          moduleName.c_str());
      }
      catch (InstanciationError &error) {
        WARNING("unable to load module " << moduleName << " ; "
                << "reverting to default (StdGS)" << std::endl <<
                "Message : " << error.message() << std::endl);
        return (GlobalScheduler::chooseGlobalScheduler());
      }
    } else
#endif
      /***********************************/
      {
        WARNING("unable to deserialize global scheduler ; "
                << "reverting to default (StdGS)");
        WARNING("scheduler was \""
                << serializedScheduler << "\"" << std::endl);
        return (GlobalScheduler::chooseGlobalScheduler());
      }
}

/** Return the serialized global scheduler (a string). */
char*
GlobalScheduler::serialize(GlobalScheduler* GS)
{
    SCHED_TRACE_FUNCTION(GS->name);

    if (!strncmp(GS->name, StdGS::stName, GS->nameLength)) {
	return StdGS::serialize((StdGS*) GS);
    }
    else if (!strncmp(GS->name, PriorityGS::stName, GS->nameLength)) {
	return PriorityGS::serialize((PriorityGS*) GS);
    }
/* New : For scheduler load support. */
#ifdef USERSCHED
    else if (!strncmp(GS->name, UserScheduler::stName, GS->nameLength)) {
	return UserScheduler::serialize(GS);
    }
#endif
    /*************************************/
    else {
	ERROR("unable to serialize global scheduler named " << GS->name, NULL);
    }
}

/** Returns a global scheduler adapted to the request. */
GlobalScheduler*
GlobalScheduler::chooseGlobalScheduler()
{
    StdGS* res = new StdGS();
    res->init();
    return res;
}

/* Let the corba_request be a parameter in order to have the possibility
   of a globalScheduler being function for example from where the request
   comes from, etc.
*/

GlobalScheduler*
GlobalScheduler::chooseGlobalScheduler(const corba_request_t* req,
                                       const corba_profile_desc_t* profile)
{
  corba_aggregator_desc_t agg = profile->aggregator;

  /* New : For scheduler load support. */
#ifdef USERSCHED
  GlobalScheduler * loaded;
#endif
  /*************************************/

  switch (agg.agg_specific._d()) {
  case DIET_AGG_DEFAULT:
    return (GlobalScheduler::chooseGlobalScheduler());
    break;
  case DIET_AGG_PRIORITY: {
    PriorityGS* ps = new PriorityGS(agg.agg_specific.agg_priority());
    ps->init();
    return (ps);
  }
    break;
    /* New : For scheduler load support. */
#ifdef USERSCHED
  case DIET_AGG_USER: {
    std::string moduleName;

    if (!CONFIG_STRING(diet::MODULENAME, moduleName)) {
      WARNING("moduleName parameter is not set in the configuration file ; "
              << "reverting to default (StdGS)" << std::endl);
      return (GlobalScheduler::chooseGlobalScheduler());
    }
    try {
      loaded = UserScheduler::instanciate(moduleName.c_str());
    }
    catch (InstanciationError &error) {
      WARNING("unable to load module " << moduleName << " ; "
              << "reverting to default (StdGS)" << std::endl
              << "Error : " << error.message() << std::endl);
      return (GlobalScheduler::chooseGlobalScheduler());
    }
    SCHED_TRACE_FUNCTION("Module " << moduleName << " loaded.");
    return loaded;
  }
    break;
#endif
    /*************************************/
  }
  ERROR(__FUNCTION__ <<
        ": unhandled aggregator (" <<
        agg.agg_specific._d() <<
        ")" << std::endl, 0);
  return 0;
}


/**
 * Aggregate and sort all servers of the \c responses. This method provides a
 * default behaviour for aggregation of sorted responses: a simple merge.
 * @param aggrResp     the aggregated response (caller-allocated).
 * @param max_srv      maximum number of servers to aggregate (all if 0)
 * @param nb_responses number of responses to aggregate.
 * @param responses    array of the responses to aggregate.
 */
int
GlobalScheduler::aggregate(corba_response_t* aggrResp, size_t max_srv,
			   const size_t nb_responses,
			   const corba_response_t* responses)
{
    size_t total_size = 0;
    // lock the schedulers
    SchedList::Iterator* iter = this->schedulers.getIterator();

    //corba_response_t* aggrResp = new corba_response_t;
    int lastAggregated = -1;
    int* lastAggr = new int[nb_responses];

    /*
    ** structure for caching estVector structures.  initially this
    ** is a vector with as many empty vectors as there are
    ** responses.  during the aggregation phase, this cache will
    ** be populated.  at the end of aggregation, this cache needs
    ** to be purged (see below).
    */
    Vector_t evCache = new_Vector();

    //   cout << "global scheduler: " << this->name << std::endl;

    SCHED_TRACE_FUNCTION("nb_responses=" << nb_responses
			 << ",max_srv=" << max_srv);

    for (size_t i = 0; i < nb_responses; i++) {
      lastAggr[i]    = -1;
      total_size    += responses[i].servers.length();
      Vector_add(evCache, new_Vector());
    }
    if (max_srv == 0)
	max_srv = total_size; // keep all servers
    aggrResp->servers.length(MIN(total_size, max_srv));

    while (iter->hasCurrent()) {
	Scheduler* sched = iter->getCurrent();

	//     fprintf(stderr,
	//             "before %s, lastAggregated=%d\n",
	//             Scheduler::serialize(sched),
	//             lastAggregated);
	sched->aggregate(*aggrResp,
			 &lastAggregated,
			 nb_responses,
			 responses,
			 lastAggr,
			 evCache);
	iter->next();
    }

    {  /* purge the estVector cache */
	while (! Vector_isEmpty(evCache)) {
	    Vector_t respV = (Vector_t) Vector_removeAtPosition(evCache, 0);
	    while (! Vector_isEmpty(respV)) {
		Vector_removeAtPosition(respV, 0);
	    }
	    free_Vector(respV);
	}
	free_Vector(evCache);
    }

    delete iter; // unlock the schedulers list
    delete [] lastAggr;

    return 0;
}


/****************************************************************************/
/* StdGS (Standard Global Scheduler)                                        */
/****************************************************************************/
#undef SCHED_CLASS
#define SCHED_CLASS "StdGS"

const char*  StdGS::stName     = "StdGS";

StdGS::StdGS()
{
    this->name = this->stName;
    this->nameLength = strlen(this->name);
}

StdGS::~StdGS() {};

  /** Initialize this global scheduler (build its list of schedulers). */
void
StdGS::init()
{
    // schedulers has already been contructed by implicit call to parent class
    // constructor.
#if !HAVE_CORI
    this->schedulers.addElement(new FASTScheduler());
    this->schedulers.addElement(new NWSScheduler(3.0, 2.0, 1.0));
#endif// !HAVE_CORI
    this->schedulers.addElement(new RRScheduler());
    this->schedulers.addElement(new RandScheduler());
}

StdGS*
StdGS::deserialize(const char* serializedScheduler)
{
    char* token(0);
    char* ser_sched = strdup(serializedScheduler);
    char* ptr = ser_sched;
    StdGS* res = new StdGS();

    TRACE_TEXT(TRACE_ALL_STEPS,
	       "StdGS::deserialize(" << serializedScheduler << ")" << std::endl);
    token = strsep( &ptr, ":" );
    assert(!strcmp(token, StdGS::stName));
    while (ptr) { // ptr == NULL when the last token is identified (no more ':')
       // If the string was not duplicated in this function, we should
       // reset the delimiter with
	// ptr[-1] = ':';
	token = strsep( &ptr, ":" );
	res->schedulers.addElement(Scheduler::deserialize(token));
    }
    free(ser_sched);
    return res;
}

char*
StdGS::serialize(StdGS* GS)
{
    size_t maxLength = 128;
    char* res = new char[maxLength];
    SchedList::Iterator* iter = GS->schedulers.getIterator();
    size_t length = GS->nameLength;

    SCHED_TRACE_FUNCTION(GS->name);
    sprintf(res, "%s", GS->name);
    while (iter->hasCurrent()) {
	Scheduler* sched = iter->getCurrent();
	char* tmp = Scheduler::serialize(sched);
	size_t tmp_length = strlen(tmp) + 1; // Add 1 for the ':'
	if ((length + tmp_length) >= maxLength) {
	    maxLength += 128;
	    char* new_res = new char[maxLength];
	    sprintf(new_res, "%s:%s", res, tmp);
	    delete [] res;
	    res = new_res;
	} else {
	    sprintf((char*)(res+length), ":%s", tmp);
	}
	length += tmp_length;
	delete [] tmp;
	iter->next();
    }
    delete iter;
    return res;
}



/****************************************************************************/
/* PriorityGS (Priority List Global Scheduler)                              */
/****************************************************************************/
#undef SCHED_CLASS
#define SCHED_CLASS "PriorityGS"

const char*  PriorityGS::stName     = "PriorityGS";

PriorityGS::PriorityGS()
{
    this->name = this->stName;
    this->nameLength = strlen(this->name);
}
PriorityGS::PriorityGS(corba_agg_priority_t priority)
{
    this->name = this->stName;
    this->nameLength = strlen(this->name);
    this->numPValues = priority.priorityList.length();
    this->pValues = new int[this->numPValues];
    for (int pvIter = 0 ; pvIter < this->numPValues ; pvIter++) {
	this->pValues[pvIter] = priority.priorityList[pvIter];
    }
}

PriorityGS::~PriorityGS() {};

  /** Initialize this global scheduler (build its list of schedulers). */
void
PriorityGS::init()
{
    this->schedulers.addElement(new PriorityScheduler(this->numPValues,
						      this->pValues));
}

PriorityGS*
PriorityGS::deserialize(const char* serializedScheduler)
{
    char* token;
    // duplicate the string because of CONST specifier in argument
    char* ser_sched = strdup(serializedScheduler);
    char* ptr = ser_sched;
    PriorityGS* res = new PriorityGS();

    TRACE_TEXT(TRACE_ALL_STEPS,
	       "PriorityGS::deserialize("
	       << serializedScheduler << ")" << std::endl);

    // Eliminate the first token, which is to be stName
    token = strsep( &ptr, ":" );
    assert(!strcmp(token, PriorityGS::stName));

    // Then for each token add associated scheduler
    while (ptr) { // ptr == NULL when the last token is identified (no more ':')
	// If the string was not duplicated in this function, we should
	// reset the delimiter with
	// ptr[-1] = ':';
	token = strsep( &ptr, ":" );
	res->schedulers.addElement(Scheduler::deserialize(token));
    }
    free(ser_sched);
    return res;
}

char*
PriorityGS::serialize(PriorityGS* GS)
{
    size_t maxLength = 128;
    char* res = new char[maxLength];
    SchedList::Iterator* iter = GS->schedulers.getIterator();
    size_t length = GS->nameLength;

    SCHED_TRACE_FUNCTION(GS->name);
    sprintf(res, "%s", GS->name);
    //   cout << "res is " << res << std::endl;
    while (iter->hasCurrent()) {
	Scheduler* sched = iter->getCurrent();
	char* tmp = Scheduler::serialize(sched);
	size_t tmp_length = strlen(tmp) + 1; // Add 1 for the ':'
	if ((length + tmp_length) >= maxLength) {
	    maxLength += 128;
	    char* new_res = new char[maxLength];
	    sprintf(new_res, "%s:%s", res, tmp);
	    delete [] res;
	    res = new_res;
	} else {
	    sprintf((char*)(res+length), ":%s", tmp);
	}
	//     cout << "  res is " << res << std::endl;
	length += tmp_length;
	delete [] tmp;
	iter->next();
    }
    delete iter;
    return res;
}
