/****************************************************************************/
/* DIET Data Location Manager implementation source code                    */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Bruno DEL FABBRO (Bruno.DelFabbro@lifc.univ-fcomte.fr)              */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.22  2011/02/04 15:20:48  hguemar
 * fixes to new configuration parser
 * some cleaning
 *
 * Revision 1.21  2011/01/21 16:37:50  bdepardo
 * Prefer prefix ++/-- operators for non-primitive types.
 *
 * Revision 1.20  2010/07/12 16:14:11  glemahec
 * DIET 2.5 beta 1 - Use the new ORB manager and allow the use of SSH-forwarders for all DIET CORBA objects
 *
 * Revision 1.19  2010/03/03 10:19:03  bdepardo
 * Changed \n into endl
 *
 * Revision 1.18  2008/06/01 14:06:56  rbolze
 * replace most ot the cout by adapted function from debug.cc
 * there are some left ...
 *
 * Revision 1.17  2007/07/13 10:00:25  ecaron
 * Remove deprecated code (ALTPREDICT part)
 *
 * Revision 1.16  2006/11/16 09:55:54  eboix
 *   DIET_config.h is no longer used. --- Injay2461
 *
 *
 * Revision 1.14  2005/09/05 16:04:10  hdail
 * Movement of subtree portion of whereData into separate method call so it can
 * also be used without searching higher in tree.
 *
 * Revision 1.13  2005/06/28 15:56:56  hdail
 * Changing the debug level of messages to make DIET less verbose (and in
 * agreement with the doc =).
 *
 * Revision 1.12  2005/04/13 08:49:11  hdail
 * Beginning of adoption of new persistency model: DTM is enabled by default and
 * JuxMem will be supported via configure flags.  DIET will always provide at
 * least one type of persistency.  As a first step, persistency across DTM and
 * JuxMem is not supported so all persistency handling should be surrounded by
 *     #if HAVE_JUXMEM
 *         // JuxMem code
 *     #else
 *         // DTM code
 *     #endif
 * This check-in prepares for the JuxMem check-in by cleaning up old
 * DEVELOPPING_DATA_PERSISTENCY flags and surrounding DTM code with
 * #if ! HAVE_JUXMEM / #endif flags to be replaced by above format by Mathieu's
 * check-in.  Currently the HAVE_JUXMEM flag is set in AgentImpl.hh - to be
 * replaced by Mathieu's check-in of a configure system for JuxMem.
 *
 * Revision 1.11  2004/10/15 13:03:16  bdelfabr
 * set_data_arg method modified to manage non correct id
 *
 * Revision 1.10  2004/09/13 14:12:19  hdail
 * Cleaned up memory management for class variables myName and localHostName.
 *
 * Revision 1.9  2004/03/03 09:04:57  bdelfabr
 * add thread safe management to list
 *
 * Revision 1.8  2004/02/27 10:25:39  bdelfabr
 * coding standard
 *
 * Revision 1.7  2003/11/10 14:04:59  bdelfabr
 * add methods invoked by DM for data transfer management
 *
 * Revision 1.6  2003/10/21 13:28:26  bdelfabr
 * set persistence flag to 0
 *
 * Revision 1.5  2003/10/14 20:27:16  bdelfabr
 * adding methods for demo RNTL (print List of persistent data)
 *
 * Revision 1.3  2003/09/30 15:10:15  bdelfabr
 * applying coding standard.
 * WhereData is modified to avoid warning while compiling
 *
 * Revision 1.2  2003/09/24 09:13:05  pcombes
 * DataMgr does not need a name: use its reference.
 *
 * Revision 1.1  2003/09/22 21:07:52  pcombes
 * Set all the modules and their interfaces for data persistency.
 ***************************************************************************/

#include <unistd.h>  // For gethostname()
#include "LocMgrImpl.hh"

#include "configuration.hh"
#include "Counter.hh"
#include "DataMgr.hh"
#include "debug.hh"
#include "ms_function.hh"
#include "ORBMgr.hh"
#include "ts_container/ts_map.hh"


#define MAX_HOSTNAME_LENGTH 256

/** The trace level. */
extern unsigned int TRACE_LEVEL;


LocMgrImpl::LocMgrImpl() 
{
    this->childID           = -1;
    this->parent            = LocMgr::_nil();
    this->localHostName     = 0;
    this->myName            = 0;
    this->childIDCounter    = 0;
    this->nbLocMgrChildren  = 0;
    this->nbDataMgrChildren = 0;
    this->dataLocList.clear();
}

LocMgrImpl::~LocMgrImpl() 
{
    this->dataLocList.clear();
}

/**
 * Launch this data location manager
 * (initialization + registration in the hierarchy).
 */
int 
LocMgrImpl::run()
{
    const std::string& name;
    const std::string& parentName;

    /* Set host name */
    this->localHostName = new char[MAX_HOSTNAME_LENGTH];
    if (gethostname(this->localHostName, MAX_HOSTNAME_LENGTH)) {
	ERROR("could not get hostname", 1);
    }
    this->localHostName[MAX_HOSTNAME_LENGTH-1] = '\0';
    
    /* Bind this LocMgr to its name in the CORBA Naming Service */
    name = CONFIG(diet::NAME);
    if (name.empty())
	return 1;
    this->myName = new char[name.length() + 4]; // 3 for "Loc" + 1 for \0
    strcpy(this->myName, name.c_str());
    
    // FIXME : rewrite strcat to ms_strcat
    // FIXME: is it really necessary to do **TWO** strcpy ?
    strcat(strcpy(this->myName, name.c_str()), "Loc");
    
    /*if (ORBMgr::bindObjToName(_this(), ORBMgr::LOCMGR, this->myName)) {
      ERROR("LocMgr: could not declare myself as " << this->myName, 1);
      }*/
    ORBMgr::getMgr()->bind(LOCMGRCTXT, myName, _this());
    ORBMgr::getMgr()->fwdsBind(LOCMGRCTXT, myName,
			       ORBMgr::getMgr()->getIOR(_this()));

    /* Get the parent reference */
    name = CONFIG(diet::PARENTNAME);
    if (!name.empty()) {
	parentName = new char[strlen(name) + 4]; // 3 for "Loc" + 1 for \0
	strcpy(parentName, name);
	strcat(parentName, "Loc");
	parent =
	    /*LocMgr::_duplicate(LocMgr::_narrow(ORBMgr::getObjReference(ORBMgr::LOCMGR,
								 parentName)));*/
	    ORBMgr::getMgr()->resolve<LocMgr, LocMgr_ptr>(LOCMGRCTXT, parentName);
	if (CORBA::is_nil(parent)) {
	    ERROR("LocMgr: cannot locate my parent " << parentName, 1);
	}
    }
    TRACE_TEXT(TRACE_ALL_STEPS,
	       endl << "Root LocMgr " 
	       << this->myName<< " started." 
	       << endl << endl);
    return 0;
}




/****************************************************************************/
/* Subscribing                                                              */
/****************************************************************************/

/** Subscribe a LocMgr as a child. Remotely called by an LocMgr. */
CORBA::ULong
LocMgrImpl::locMgrSubscribe(const char* name, const char* hostName)
{
    LocMgr_ptr me = ORBMgr::getMgr()->resolve<LocMgr, LocMgr_ptr>(LOCMGRCTXT,
								  name);
    CORBA::ULong retID = (this->childIDCounter)++; // thread safe

    /* the size of the list is childIDCounter+1 (first index is 0) */
    this->locMgrChildren.resize(this->childIDCounter);
    this->locMgrChildren[retID] = locMgrChild(me, hostName);
    (this->nbLocMgrChildren)++; // thread safe
    
    if (! CORBA::is_nil(this->parent)) { // LocMgr associated to an MA.
	if (this->childID == (ChildID)-1)
	    this->childID = 
		this->parent->locMgrSubscribe(myName,
					      this->localHostName);
    }

    return retID;
} // locMgrSubscribe(...)


/** Subscribe a DataMgr as a child. Remotely called by a DataMgr. */
CORBA::ULong
LocMgrImpl::dataMgrSubscribe(const char* name, const char* hostName)
{
    DataMgr_ptr me = 
	ORBMgr::getMgr()->resolve<DataMgr, DataMgr_ptr>(DATAMGRCTXT,
							name);
    CORBA::ULong retID = (this->childIDCounter)++; // thread safe

    /* the size of the list is childIDCounter+1 (first index is 0) */
    this->dataMgrChildren.resize(this->childIDCounter);
    this->dataMgrChildren[retID] = dataMgrChild(me, hostName);
    (this->nbDataMgrChildren)++; // thread safe

    if (! CORBA::is_nil(this->parent)) { // LocMgr associated to an MA.
	if (this->childID == (ChildID)-1)
	    this->childID = 
		this->parent->locMgrSubscribe(myName, this->localHostName);
    }

    return(retID);
} // dataMgrSubscribe(...)



/****************************************************************************/
/* Manipulating data                                                        */
/****************************************************************************/


/** Add Data Reference to the Reference List */
void
LocMgrImpl::addDataRef(const corba_data_desc_t& arg, CORBA::ULong cChildID)
{
    store_desc_child_t &data_to_store = dataLocList[strdup(arg.id.idNumber)];
    data_to_store.childID_owner = cChildID;
    data_to_store.id_handle_type = arg;
    if (!CORBA::is_nil(this->parent)) {
	parent->addDataRef(arg,this->childID);
    }
}

/** initially invoked by the MA : client wants to destroy data */
CORBA::Long
LocMgrImpl::rm_pdata(const char* argID)
{
    if ( dataLocList.size() > 0) {
	store_desc_child_t &data_to_store = dataLocList[ms_strdup(argID)];
	ChildID cChildID = data_to_store.childID_owner;
	dataLocList.erase(ms_strdup(argID));
	printList1();
	if(cChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
	    locMgrChild theLoc = locMgrChildren[cChildID];
	    if (theLoc.defined()){
		LocMgr_ptr locChild = theLoc.getIor();
		return locChild->rm_pdata(argID);
	    } else return 0;
	} else {
	    if(cChildID < static_cast<CORBA::Long>(dataMgrChildren.size())) {
		dataMgrChild theData = dataMgrChildren[cChildID];
		if(theData.defined()){
	    DataMgr_ptr dataChild = theData.getIor();
	    return dataChild->rmDataRefDataMgr(ms_strdup(argID));
		} else return 0;
	    } else return 0;
	}
    } else {
	return 0; 
    }
}


/** Remove Data Reference from the Reference List */
void
LocMgrImpl::rmDataRefLocMgr(const char *argID, CORBA::ULong cChildID)
{
    dataLocList.erase(strdup(argID));
    printList1();
    if (this->parent != LocMgr::_nil()) {
	parent->rmDataRefLocMgr(argID,cChildID);
    }
}

/**
 * initially invoked by a Data Manager.
 * the owner a the data has changed. The list must be updated.
 */ 
void
LocMgrImpl::updateDataRef(const corba_data_desc_t& arg,
			  CORBA::ULong cChildID, CORBA::Long upDown)
{
    if(upDown == 0){ // UP = replace old owner by new one
	if ( CORBA::is_nil(this->parent)) { // Root Loc Manager

	    store_desc_child_t &stored_desc = 
		dataLocList[ms_strdup(arg.id.idNumber)];
	    
	    ChildID oldChildID = stored_desc.childID_owner;
	    stored_desc.childID_owner = cChildID;

	    dataLocList[ms_strdup(arg.id.idNumber)]=stored_desc;
	    printList1();

	    upDown = 1;
	    if(oldChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
		locMgrChild theLoc = locMgrChildren[oldChildID];
		if (theLoc.defined()){
		    LocMgr_ptr locChild = theLoc.getIor();
		    locChild->updateDataRef(arg,(unsigned)-1,1);
		}
	    }
	    if(oldChildID < static_cast<CORBA::Long>(dataMgrChildren.size())) {
		dataMgrChild theData = dataMgrChildren[oldChildID];
		if(theData.defined()){
		    DataMgr_ptr dataChild = theData.getIor();
		    dataChild->rmDataRefDataMgr(ms_strdup(arg.id.idNumber));
		}
	    }
	} else {
	    // Not Root Loc Manager
	    // data not found - adding it
	    if(dataLookUp(ms_strdup(arg.id.idNumber)) == 1) { 
		store_desc_child_t stored_desc ;
		stored_desc.childID_owner = cChildID;
		stored_desc.id_handle_type = arg;
		dataLocList[ms_strdup(arg.id.idNumber)] = stored_desc;
		printList1();
	parent->updateDataRef(arg, this->childID,0); 
	    } else {
		store_desc_child_t &stored_desc =
		    dataLocList[ms_strdup(arg.id.idNumber)];
		ChildID oldChildID = stored_desc.childID_owner;
		stored_desc.childID_owner = cChildID;
		dataLocList[ms_strdup(arg.id.idNumber)] = stored_desc;

		upDown = 1;
		if(oldChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
		    locMgrChild theLoc = locMgrChildren[oldChildID]; 
		    if (theLoc.defined()){
			LocMgr_ptr locChild = theLoc.getIor();
			locChild->updateDataRef(arg,(unsigned)-1,1);
		    }
		}
		
		if(oldChildID < 
		       static_cast<CORBA::Long>(dataMgrChildren.size())) {
		    dataMgrChild theData = dataMgrChildren[oldChildID];	  
		    if(theData.defined()){
			DataMgr_ptr dataChild = theData.getIor();
			dataChild->rmDataRefDataMgr(ms_strdup(arg.id.idNumber));
		    }
		}
	    }
	}
    } else { // DOWN = removing old owner
	if (!CORBA::is_nil(this->parent)){
	    store_desc_child_t &stored_desc =
		dataLocList[ms_strdup(arg.id.idNumber)];
	    ChildID oldChildID = stored_desc.childID_owner;
	    stored_desc.childID_owner = cChildID;
	    dataLocList.erase(strdup(arg.id.idNumber));
	    printList1();
	    if(oldChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
		locMgrChild theLoc = locMgrChildren[oldChildID];
		if (theLoc.defined()){
		    LocMgr_ptr locChild = theLoc.getIor();
		    locChild->updateDataRef(arg,(unsigned)-1,1);
		}
	    }
	    if(oldChildID < static_cast<CORBA::Long>(dataMgrChildren.size())) {
		dataMgrChild theData = dataMgrChildren[oldChildID];
		if (theData.defined()){
		    DataMgr_ptr dataChild = theData.getIor();
		    dataChild->rmDataRefDataMgr(arg.id.idNumber);
		}
	    }
	}
    } 
} //updateDataRef(const corba_data_desc_t& arg, CORBA::ULong cChildID, CORBA::Long upDown)


/**
 * Returns the Id of :
 *   - the Data Manager holding the Data referenced by its id OR
 *   - the Loc Manager holding the reference of the Data
 */
CORBA::ULong
LocMgrImpl::dataLookUp(const char *argID)
{
    dataLocList.lock();
    dataLocList.begin();
    if(dataLocList.find(strdup(argID)) != dataLocList.end()){
	dataLocList.unlock();
	return 0;
    } else {
	dataLocList.unlock();
	return 1;
    }
} //dataLookUp(const char *argID)


/** get data properties (type and data type) */
corba_data_desc_t*
LocMgrImpl::set_data_arg(const char* argID)
{
    corba_data_desc_t *arg_data_desc = new corba_data_desc_t;

    //printList1();
    if( dataLookUp(argID) == 1 ) {
	arg_data_desc->id.idNumber = CORBA::string_dup("-1");
	arg_data_desc->id.dataCopy =  DIET_ORIGINAL;
	arg_data_desc->id.state = DIET_FREE;
    } else {
	store_desc_child_t stored_desc = dataLocList[ms_strdup(argID)];
	*arg_data_desc = stored_desc.id_handle_type;
    }

    return arg_data_desc;
} // set_data_arg(const char* argID)


/** Display the List of known data Reference */
char *
LocMgrImpl::setMyName(){
    return CORBA::string_dup(this->myName);
}

void
LocMgrImpl::printList1()
{
    char *SonName = 0;

    if( dataLocList.size() > 0){
	TRACE_TEXT(TRACE_ALL_STEPS,
		   "+------------+----------------------------+" << std::endl <<
		   "| Data Name  | Data Manager Owner         |" << std::endl <<
		   "+------------+----------------------------+" << std::endl);
	dataLocList.lock();
	DataLocList_t::iterator cur = dataLocList.begin();
	while (cur != dataLocList.end()) {
	    if(cur->second.childID_owner < 
		   static_cast<CORBA::Long>(locMgrChildren.size())) {
		locMgrChild theLoc = locMgrChildren[cur->second.childID_owner];
		if (theLoc.defined()) {
		    LocMgr_ptr locChild = theLoc.getIor();
		    SonName = locChild->setMyName();
		}
	    } else {
		dataMgrChild theData = dataMgrChildren[cur->second.childID_owner];
		if (theData.defined()) {
		    DataMgr_ptr dataChild = theData.getIor();
		    SonName = dataChild->setMyName();
		}
	    }
	    TRACE_TEXT(TRACE_ALL_STEPS,"| " 
		       << cur->first << " |   " << SonName  << std::endl);
	    TRACE_TEXT(TRACE_ALL_STEPS,"+------------+" << std::endl); 
	    ++cur;
	}
	dataLocList.unlock();
	TRACE_TEXT(TRACE_ALL_STEPS,
		   "+------------+----------------------------+" << std::endl); 
    } /*else {
	cout << "+-----No Data-----+" << std::endl;
	}*/
} // printList1()


void
LocMgrImpl::printList()
{
    printList1();
    if (this->parent != LocMgr::_nil()) {
	this->parent->printList();
    }
} // printList()

/**
 * returns the name of the DataManager that owns a data identified by argID,
 * NULL if data not present, prints message if data not known  
 */
char *
LocMgrImpl::whichSeDOwner(const char* argID)
{
    if (this->parent == LocMgr::_nil()) {
	dataLocList.lock();
	dataLocList.begin();
	if (dataLocList.find(strdup(argID)) != dataLocList.end()) { 
	    dataLocList.unlock();
	    store_desc_child_t &stored_desc = dataLocList[ms_strdup(argID)];
	    ChildID cChildID = stored_desc.childID_owner;

	    if(cChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
		locMgrChild theLoc = locMgrChildren[cChildID];
		if (theLoc.defined()) {
		    LocMgr_ptr locChild = theLoc.getIor();
		    return locChild->whichSeDOwner(ms_strdup(argID)) ;
		} else return 0;
	    } else {
		if(cChildID < static_cast<CORBA::Long>(dataMgrChildren.size())) {
		    dataMgrChild theData = dataMgrChildren[cChildID];
		    if (theData.defined()) {
			DataMgr_ptr dataChild = theData.getIor();
			return dataChild->whichSeDOwner(ms_strdup(argID)) ;
		    } else return 0;
		} else {
		    cerr << "OBJECT UNKNOWN " << std::endl;
		    return 0;
		}
	    }
	} else {
	    dataLocList.unlock();
	    cerr << "DATA NOT IN HIERARCHY " << std::endl;
	    return 0;
	}
	
    } else { /** I am not root Loc Manager */ 
	dataLocList.lock();
	dataLocList.begin();
	/** Data Reference found */
	if (dataLocList.find(strdup(argID)) != dataLocList.end()) {
	    dataLocList.unlock();
	    store_desc_child_t &stored_desc = dataLocList[ms_strdup(argID)];
	    ChildID cChildID = stored_desc.childID_owner;
	    if(cChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
		locMgrChild theLoc = locMgrChildren[cChildID];
		if (theLoc.defined()){
		    LocMgr_ptr locChild = theLoc.getIor();
		    return locChild->whichSeDOwner(ms_strdup(argID)) ;
		} else return 0;
	    } else {
		if(cChildID < static_cast<CORBA::Long>(dataMgrChildren.size())) {
		    dataMgrChild theData = dataMgrChildren[cChildID];
		    if (theData.defined()){
			DataMgr_ptr dataChild = theData.getIor();
			return dataChild->whichSeDOwner(ms_strdup(argID));
		    } else return 0;
		} else {
		    return 0; 
		}
	    }
	} else {
	    /** data Reference not found - invoke my parent */
	    dataLocList.unlock();
	    return this->parent->whichSeDOwner(ms_strdup(argID));
	}
    }
} // whichSeDOwner(const char* argID)

 
/**
 * Returns the IOR of the Data Manager holding the searched data
 */
char*
LocMgrImpl::whereData(const char* argID)
{
    /** Check first if data exists locally or in subtree */
    char* dataMgrRef = this->whereDataSubtree(argID);

    if (strcmp(dataMgrRef, "") == 0) {
	dataLocList.lock();
	dataLocList.begin();

	/** Not found.  Try looking to parent if not the root node */
	if (this->parent == LocMgr::_nil()) {
	    /** I am the root Loc Manager */
	    dataLocList.unlock();
	    WARNING("Data item " 
		    << argID << " not found in hierarchy" << std::endl);
	    dataMgrRef = CORBA::string_dup("");
	} else {
	    /** I am not the root.  Try asking my parent. */
	    dataLocList.unlock();
	    dataMgrRef = this->parent->whereData(ms_strdup(argID));
	}
    }
    return dataMgrRef;
} // whereData(const char* argID)

/**
 * Returns the reference to the Data Manager holding the needed data.
 * Only search for the data locally and in the subtree under this LocMgr.
 */
char*
LocMgrImpl::whereDataSubtree(const char* argID)
{
    dataLocList.lock();
    dataLocList.begin();

    if (dataLocList.find(strdup(argID)) != dataLocList.end()) {
	/** Data location found */
	dataLocList.unlock();
	store_desc_child_t &stored_desc = dataLocList[ms_strdup(argID)];
	ChildID cChildID = stored_desc.childID_owner;

	if(cChildID < static_cast<CORBA::Long>(locMgrChildren.size())) {
	    /** Data available with a LocMgr */
	    locMgrChild theLoc =  locMgrChildren[cChildID];
	    if (theLoc.defined()) {
		LocMgr_ptr locChild = theLoc.getIor();
		return locChild->whereDataSubtree(ms_strdup(argID)) ;
	    } else {
		return CORBA::string_dup("");
	    }
	    
	} else {
	    /** Data available with a dataMgr */
	    if(cChildID < static_cast<CORBA::Long>(dataMgrChildren.size())) {
		dataMgrChild theData = dataMgrChildren[cChildID];
		if (theData.defined()) {
		    DataMgr_ptr dataChild = theData.getIor();
		    return dataChild->whereData(ms_strdup(argID)) ;
		} else {
		    return CORBA::string_dup("");
		}
	    } else {
		WARNING("Unknown object cChildID" << std::endl);
		return CORBA::string_dup("");
	    }
	}
    } else {
	dataLocList.unlock();
	TRACE_TEXT(TRACE_STRUCTURES, "Data " 
		   << argID << " not in subhierarchy" << std::endl);
	return CORBA::string_dup("");
    }
} // whereDataSubtree(const char* argID)


void
LocMgrImpl::updateDataProp(const char *argID)
{
    if (this->parent == LocMgr::_nil()) {
	this->parent->updateDataProp(ms_strdup(argID));
    }
}

// Forwarders part
LocMgrFwdrImpl::LocMgrFwdrImpl(Forwarder_ptr fwdr, const char* objName)
{
    this->forwarder = Forwarder::_duplicate(fwdr);
    this->objName = CORBA::string_dup(objName);
}

::CORBA::ULong LocMgrFwdrImpl::locMgrSubscribe(const char* me, 
					       const char* hostName)
{
    return forwarder->locMgrSubscribe(me, hostName, objName);
}

::CORBA::ULong LocMgrFwdrImpl::dataMgrSubscribe(const char* me,
						const char* hostName)
{
    return forwarder->dataMgrSubscribe(me, hostName, objName);
}

void LocMgrFwdrImpl::addDataRef(const ::corba_data_desc_t& arg,
				::CORBA::ULong cChildID)
{
    forwarder->addDataRef(arg, cChildID, objName);
}

void LocMgrFwdrImpl::rmDataRefLocMgr(const char* argID,
				     ::CORBA::ULong cChildID)
{
    forwarder->rmDataRefLocMgr(argID, cChildID, objName);
}

void LocMgrFwdrImpl::updateDataRef(const ::corba_data_desc_t& arg,
				   ::CORBA::ULong cChildID,
				   ::CORBA::Long upDown)
{
    forwarder->updateDataRef(arg, cChildID, upDown, objName);
}

char* LocMgrFwdrImpl::whereData(const char* argID)
{
    return forwarder->whereData(argID, objName);
}

char* LocMgrFwdrImpl::whereDataSubtree(const char* argID)
{
    return forwarder->whereDataSubtree(argID, objName);
}

void LocMgrFwdrImpl::updateDataProp(const char* argID)
{
    forwarder->updateDataProp(argID, objName);
}

::CORBA::Long LocMgrFwdrImpl::rm_pdata(const char* argID)
{
    return forwarder->rm_pdata(argID, objName);
}

char* LocMgrFwdrImpl::setMyName()
{
    return forwarder->setMyName(objName);
}

char* LocMgrFwdrImpl::whichSeDOwner(const char* argID)
{
    return forwarder->whichSeDOwner(argID, objName);
}

void LocMgrFwdrImpl::printList()
{
    forwarder->printList(objName);
}
