/****************************************************************************/
/* A thread which updates the MultiMA links                                 */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Sylvain DAHAN (Sylvain.Dahan@lifc.univ-fcomte.fr)                   */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.3  2006/11/16 09:55:54  eboix
 *   DIET_config.h is no longer used. --- Injay2461
 *
 * Revision 1.2  2004/10/04 09:40:43  sdahan
 * warning fix :
 *  - debug.cc : change the printf format from %ul to %lu and from %l to %ld
 *  - ReferenceUpdateThread and BindService : The omniORB documentation said that
 *    it's better to create private destructor for the thread subclasses. But
 *    private destructors generate warning, so I set the destructors public.
 *  - CORBA.h and DIET_config.h define the same macros. So I include the CORBA.h
 *    before the DIET_config.h to avoid to define two times the same macros.
 *  - remove the deprecated warning when including iostream.h and set.h
 *
 * Revision 1.1  2004/09/29 13:35:31  sdahan
 * Add the Multi-MAs feature.
 *
 */


#ifndef _REFERENCE_UPDATE_THREAD_HH_
#define _REFERENCE_UPDATE_THREAD_HH_

#ifdef HAVE_MULTI_MA

#include "omnithread.h"

class MasterAgentImpl ;

/**
 * This is an thead object that take a MasterAgent_impl in
 * argument. Each periods of time, it calls the updateRef of this
 * agent.
 *
 * @author Sylvain DAHAN - LIFC Besancon (France)
 */

class ReferenceUpdateThread : public omni_thread {

private :

  /**
   * A pointer on the master agent that own this thread. This is this
   * agent which are updated each 5 minutes.
   */
  MasterAgentImpl* owner ;

  /**
   * amount of time that it wait between two references updates.
   */
  unsigned int period ;

  /**
   * Updates the references of the owner, waits 5 minutes and do it
   * again.
   */
  void run(void* ptr) ;

public :
  /**
   * Creates and launch a new thread which update the reference of
   * owner each 5 minutes.
   *
   * @param owner A pointer on the master agent that own this
   * thread. This is this agent which are updated each 5 minutes.
   *
   * @param period amount of time that it wait between two references
   * updates.
   */
  ReferenceUpdateThread(MasterAgentImpl* owner, unsigned int period) ;

  /**
   * destroys the thread.
   */
  ~ReferenceUpdateThread() ;

} ; // ReferenceUpdateThread

#endif // HAVE_MULTI_MA

#endif // _REFERENCE_UPDATE_THREAD_HH_
