/****************************************************************************/
/* Another multi-workflow scheduler                                         */
/* This scheduler is based on First Come First Serve                        */
/*                                                                          */
/* Author(s):                                                               */
/* - Raphaël  Bolze  (raphael.bolze@ens-lyon.fr)                            */
/* - Benjamin ISNARD (benjamin.isnard@ens-lyon.fr)                          */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.3  2009/01/16 13:41:22  bisnard
 * added common base class DagScheduler to simplify dag events handling
 * improved exception management
 *
 * Revision 1.2  2008/10/14 13:24:49  bisnard
 * use new class structure for dags (DagNode,DagNodePort)
 *
 * Revision 1.1  2008/07/24 21:08:11  rbolze
 * New multi-wf heuristic FCFS (First Come First Serve)
 *
 *
 */

#ifndef _MULTIWFFCFC_HH_
#define _MULTIWFFCFS_HH_

#include "MultiWfScheduler.hh"

using namespace std;

namespace madag {

  class MultiWfFCFS : public MultiWfScheduler {
  public:
    MultiWfFCFS(MaDag_impl* maDag);
    virtual ~MultiWfFCFS();

  protected:

    /**
     * Updates scheduler when a node has been executed
     */
    virtual void
        handlerNodeDone(DagNode * node);

    /**
     * set node priority before inserting into execution queue
     */
    virtual void
        setExecPriority(DagNode * node);

    /**
     * set node priority before inserting back in the ready queue
     */
    virtual void
        setWaitingPriority(DagNode * node);
  };

}

#endif   /* not defined _MULTIWFFCFS_HH */

