/****************************************************************************/
/* POA callback implementation                                             */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Christophe PERA (christophe.pera@ens-lyon.fr)                       */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.10  2011/04/21 16:02:59  bdepardo
 * Code indentation
 *
 * Revision 1.9  2010/07/12 16:14:11  glemahec
 * DIET 2.5 beta 1 - Use the new ORB manager and allow the use of SSH-forwarders for all DIET CORBA objects
 *
 * Revision 1.8  2006/08/09 21:39:35  aamar
 * Notify result before setting the request error code
 *
 * Revision 1.7  2006/07/10 13:46:00  aamar
 * Correcting a warning
 *
 * Revision 1.6  2006/07/07 09:29:03  aamar
 * Modify the solveResults prototype: adding the parameter solve_res
 * that represents the execution status of the asynchronous request.
 *
 * Revision 1.5  2003/10/13 13:02:22  cpera
 * Replace long by CORBA::Long.
 *
 * Revision 1.4  2003/09/25 10:06:24  cpera
 * Delete messages and useless include.
 *
 * Revision 1.3  2003/09/16 10:26:41  cpera
 * Fix a warning.
 *
 * Revision 1.2  2003/06/02 09:00:19  cpera
 * Delete debug infos.
 *
 * Revision 1.1  2003/06/02 08:09:55  cpera
 * Beta version of asynchronize DIET API.
 *
 ****************************************************************************/

using namespace std;
#include <iostream>
#include <CallbackImpl.hh>
#define TRACE_LEVEL 50
#include "debug.hh"

#include "Forwarder.hh"
#include "CallbackFwdr.hh"

CallbackImpl::CallbackImpl() {
  // nothing
}

CallbackImpl::~CallbackImpl() {
  // nothing
}

CORBA::Long CallbackImpl::ping() {
  return 0;
}

CORBA::Long CallbackImpl::notifyResults(const char * path,
					const corba_profile_t& pb,
					CORBA::Long reqID) {
  /*
    if (solve_res != GRPC_NO_ERROR)
    CallAsyncMgr::Instance()->setReqErrorCode(reqID, solve_res);
  */
  return 0;
}

CORBA::Long
CallbackImpl::solveResults(const char * path, 
			   const corba_profile_t& pb,
			   CORBA::Long reqID,
			   CORBA::Long solve_res) {
  // notify solve result availability
  int err = CallAsyncMgr::Instance()->notifyRst(int(reqID), &(const_cast<corba_profile_t&>(pb)));
  if (solve_res != GRPC_NO_ERROR)
    CallAsyncMgr::Instance()->setReqErrorCode(reqID, solve_res);
  return err;
}

CallbackFwdrImpl::CallbackFwdrImpl(Forwarder_ptr fwdr, const char* objName) {
  this->forwarder = Forwarder::_duplicate(fwdr);
  this->objName = CORBA::string_dup(objName);
}

CORBA::Long CallbackFwdrImpl::ping() {
  return forwarder->ping(objName);
}

CORBA::Long CallbackFwdrImpl::notifyResults(const char * path,
                                            const corba_profile_t& pb,
                                            CORBA::Long reqID) {
  return forwarder->notifyResults(path, pb, reqID, objName);
}

CORBA::Long CallbackFwdrImpl::solveResults(const char * path, 
                                           const corba_profile_t& pb,
                                           CORBA::Long reqID,
                                           CORBA::Long solve_res) {
  return forwarder->solveResults(path, pb, reqID, solve_res, objName);
}
