/****************************************************************************/
/* POAcallback implementation                                              */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Christophe PERA (christophe.pera@ens-lyon.fr)                       */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.5  2010/07/12 16:14:11  glemahec
 * DIET 2.5 beta 1 - Use the new ORB manager and allow the use of SSH-forwarders for all DIET CORBA objects
 *
 * Revision 1.4  2006/07/07 09:29:03  aamar
 * Modify the solveResults prototype: adding the parameter solve_res
 * that represents the execution status of the asynchronous request.
 *
 * Revision 1.3  2003/10/13 13:02:22  cpera
 * Replace long by CORBA::Long.
 *
 * Revision 1.2  2003/06/02 14:51:54  cpera
 * Applying coding standards on Callback.idl.
 *
 * Revision 1.1  2003/06/02 08:09:55  cpera
 * Beta version of asynchronize DIET API.
 *
 ****************************************************************************/


#ifndef _CALLBACKIMPL_H_
#define _CALLBACKIMPL_H_

#include "Callback.hh"
#include "CallAsyncMgr.hh"

#include "Forwarder.hh"
#include "CallbackFwdr.hh"

class CallbackImpl : public POA_Callback,
	            public PortableServer::RefCountServantBase
{
public:
  CallbackImpl();
  virtual ~CallbackImpl();
  virtual CORBA::Long ping();
  virtual CORBA::Long notifyResults(const char * path,
				    const corba_profile_t& pb,
				    CORBA::Long reqID);
  virtual CORBA::Long solveResults(const char * path, 
				   const corba_profile_t& pb,
				   CORBA::Long reqID,
				   CORBA::Long solve_res);
};

class CallbackFwdrImpl : public POA_CallbackFwdr,
	public PortableServer::RefCountServantBase
{
private:
	Forwarder_ptr forwarder;
	char* objName;
public:
	CallbackFwdrImpl(Forwarder_ptr fwdr, const char* objName);
	virtual CORBA::Long ping();
  virtual CORBA::Long notifyResults(const char * path,
																		const corba_profile_t& pb,
																		CORBA::Long reqID);
  virtual CORBA::Long solveResults(const char * path, 
																	 const corba_profile_t& pb,
																	 CORBA::Long reqID,
																	 CORBA::Long solve_res);
};
#endif
