/****************************************************************************/
/* Client workflow manager class                                            */
/*                                                                          */
/* Author(s):                                                               */
/* - Abdelkader AMAR (Abdelkader.Amar@ens-lyon.fr)                          */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.27  2011/03/17 10:14:28  hguemar
 * clean some stuff left behind
 *
 * Revision 1.25  2011/03/16 22:18:37  bdepardo
 * Added a method to delete the unique reference of CltWfMgr
 *
 * Revision 1.24  2011/03/15 22:51:02  bdepardo
 * Code cleanup and indentation
 *
 * Revision 1.23  2010/12/17 09:48:00  kcoulomb
 * * Set diet to use the new log with forwarders
 * * Fix a CoRI problem
 * * Add library version remove DTM flag from ccmake because deprecated
 *
 * Revision 1.22  2010/07/20 09:22:33  bisnard
 * changes for dietForwarder
 *
 * Revision 1.21  2010/07/12 16:14:12  glemahec
 * DIET 2.5 beta 1 - Use the new ORB manager and allow the use of SSH-forwarders for all DIET CORBA objects
 *
 * Revision 1.20  2010/03/08 13:37:50  bisnard
 * replaced WfLogService by DietLogComponent
 *
 * Revision 1.19  2009/09/25 12:39:18  bisnard
 * modified includes to reduce inter-dependencies
 *
 * Revision 1.18  2009/08/26 10:35:24  bisnard
 * provide new methods for data input/output and execution transcript
 *
 * Revision 1.17  2009/07/23 12:26:06  bisnard
 * new API method to get functional wf results as a container
 *
 * Revision 1.16  2009/06/23 09:19:34  bisnard
 * use new classname for WfLogService
 * added missing exception catch in execNodeCommon
 *
 * Revision 1.15  2009/02/06 14:51:27  bisnard
 * - setup exceptions
 * - refactoring dag and functional wf submission
 *
 * Revision 1.14  2009/01/22 09:01:09  bisnard
 * added client method to retrieve workflow container output
 *
 * Revision 1.13  2009/01/16 16:32:26  bisnard
 * added option to specify data source file name
 *
 * Revision 1.12  2009/01/16 13:51:11  bisnard
 * corrected synchronization pbs between dag instanciation and execution
 *
 * Revision 1.11  2008/12/09 12:09:03  bisnard
 * added parameters to dag submit method to handle inter-dependent dags
 *
 * Revision 1.10  2008/12/02 14:17:48  bisnard
 * manage multi-dag cancellation when one dag fails
 *
 * Revision 1.9  2008/12/02 10:19:15  bisnard
 * functional workflow submission to MaDag
 *
 * Revision 1.8  2008/07/11 09:12:34  bisnard
 * Added exclusion blocks to avoid dag not found error
 *
 * Revision 1.7  2008/07/08 11:14:21  bisnard
 * Add dag makespan value in release message
 *
 * Revision 1.6  2008/06/26 15:00:18  bisnard
 * corba type mismatch
 *
 * Revision 1.5  2008/06/25 09:59:39  bisnard
 * new params in execNodeOnSeD to provide ReqId and estimation vector
 * to client for solve request
 *
 * Revision 1.4  2008/06/02 08:34:20  bisnard
 * Execute method (for wf node) now returns an error code in case of
 * communication failure with the SeD
 *
 * Revision 1.3  2008/06/01 09:20:37  rbolze
 * the remote method release now return a string which contains
 * feedback on the dag execution
 *
 * Revision 1.2  2008/05/16 12:32:10  bisnard
 * API function to retrieve all workflow results
 *
 * Revision 1.1  2008/04/30 07:26:10  bisnard
 * moved CltWfMgr into agent/workflow directory
 *
 * Revision 1.5  2008/04/28 12:08:16  bisnard
 * obsolete init_wf_call
 * changed constructor for Node (new param wfReqId)
 *
 * Revision 1.4  2008/04/21 14:31:45  bisnard
 * moved common multiwf routines from derived classes to MultiWfScheduler
 * use wf request identifer instead of dagid to reference client
 * use nodeQueue to manage multiwf scheduling
 * renamed WfParser as DagWfParser
 *
 * Revision 1.3  2008/04/15 14:20:20  bisnard
 * - Postpone sed mapping until wf node is executed
 *
 * Revision 1.2  2008/04/14 09:10:40  bisnard
 *  - Workflow rescheduling (CltReoMan) no longer used with MaDag v2
 *  - AbstractWfSched and derived classes no longer used with MaDag v2
 *
 * Revision 1.1  2008/04/10 08:38:50  bisnard
 * New version of the MaDag where workflow node execution is triggered by the MaDag agent and done by a new CORBA object CltWfMgr located in the client
 *
 ****************************************************************************/

#ifndef _CLTWFMGR_HH_
#define _CLTWFMGR_HH_

// STL headers
#include <map>
#include <string>

// IDL headers
#include "CltMan.hh"
#include "MasterAgent.hh"
#include "MaDag.hh"
#ifdef USE_LOG_SERVICE
#include "DietLogComponent.hh"
#endif
#include "WfLogService.hh"

// DIET headers
#include "DIET_data.h"
#include "DIET_client.h"
#include "marshalling.hh"

// Forwarder part
#include "Forwarder.hh"
#include "CltManFwdr.hh"

class NodeSet;
class FWorkflow;
class Dag;
class MetaDag;

using namespace std;

class CltWfMgr : public POA_CltMan,
                 public PortableServer::RefCountServantBase{
public:
  /**
   * Executes a node on a specified Sed (CORBA method)
   * @param node_id node identifier
   * @param dag_id  dag identifier
   * @param sed     SeD where execute service
   * @param reqID   request ID (from submit request)
   * @param ev      Estimation vector for this SeD (from submit request)
   */
  virtual CORBA::Long
  execNodeOnSed(const char * node_id,
                const char * dag_id,
                const char * sed,
                const CORBA::ULong reqID,
                corba_estimation_t& ev);

  /**
   * Executes a node without specifying the Sed (CORBA method)
   *
   * @param node_id node identifier
   * @param dag_id  dag identifier
   */
  virtual CORBA::Long
  execNode(const char * node_id, const char * dag_id);

  /**
   * (CORBA method)
   * Release the waiting semaphore. This method is used by the MA DAG when workflow execution
   * is done by this agent
   */
  virtual char *
  release(const char * dag_id, bool successful);

  /**
   * Debug function
   */
  virtual CORBA::Long ping();

  /**
   * Give access to unique reference of CltWfMgr
   */
  static CltWfMgr *
  instance();

  /**
   * Deletes the unique reference of CltWfMgr
   */
  static void terminate();

  /**
   * Set the MaDag reference
   *
   * @param maDagRef MaDag reference
   */
  void
  setMaDag(MaDag_var maDagRef);

  /**
   * Set the Master Agent reference
   *
   * @param ma Master Agent reference
   */
  void
  setMA(MasterAgent_var ma);
#ifdef USE_LOG_SERVICE
  /**
   * Set the log service reference
   *
   * @param logComponent log service reference
   */
  void
  setLogComponent(DietLogComponent* logComponent);
#endif
  /**
   * Set the workflow log service (GUI)
   * @param logService wf log service reference
   */
  void
  setWfLogService(WfLogService_var logService);

  /**
   * Get a new workflow request ID (for multi-dag submit)
   */
  CORBA::Long
  getNewWfReqID();

  /**
   * Execute a dag using the MA DAG.
   * (if the wfReqID is set in the profile, then this call is part of several
   * interdependent dag calls)
   * @param profile dag description
   */
  diet_error_t
  wfDagCall(diet_wf_desc_t * profile);

  /**
   * Execute a functional workflow using the MA DAG
   * @param profile workflow description
   */
  diet_error_t
  wfFunctionalCall(diet_wf_desc_t * profile);

  /**
   * Cancel a running dag. This method returns immediately as it only set the dag's status.
   * Dag will continue running until currently running nodes are finished, and then
   * MaDag will send the release() call.
   */
  diet_error_t
  cancelDag(const char* dagId);

  /**
   * Display all results from a dag
   */
  diet_error_t
  printAllDagResults(diet_wf_desc_t* profile);

  /**
   * DIsplay all results from a functional wf
   */
  diet_error_t
  printAllFunctionalWfResults(diet_wf_desc_t* profile);

  /**
   * Read the workflow/dag execution transcript from a file (XML)
   * (file name is stored in profile)
   * If file does not exist, method does nothing
   */
  diet_error_t
  readWorkflowExecutionTranscript(diet_wf_desc_t * profile);

  /**
   * Store the workflow/dag execution transcript in a file (XML)
   * (file name is stored in profile)
   * (If file contains sth, content is overwritten)
   */
  diet_error_t
  saveWorkflowExecutionTranscript(diet_wf_desc_t * profile,
                                  const char * transcriptFileName);

  /**
   * Store the workflow data file (XML format) after workflow execution.
   * This file can be used as input for another execution of a workflow
   * to avoid re-uploading the data items to the DIET platform (the data
   * IDs are written as attributes of the data items)
   * (if file already exists, content will be overwritten)
   */
  diet_error_t
  saveWorkflowDataFile(diet_wf_desc_t * profile,
                       const char * dataFileName);

  /**
   * Get a scalar result from a dag
   */
  int
  getWfOutputScalar(diet_wf_desc_t* profile,
                    const char * id,
                    void** value);

  /**
   * Get a string result from a dag
   */
  int
  getWfOutputString(diet_wf_desc_t* profile,
                    const char * id,
                    char** value);

  /**
   * Get a file result from a dag
   */
  int
  getWfOutputFile(diet_wf_desc_t* profile,
                  const char * id,
                  size_t* size, char** path);

  /**
   * Get a matrix result from a dag
   */
  int
  getWfOutputMatrix(diet_wf_desc_t* profile,
                    const char * id,
                    void** value,
                    size_t* nb_rows,
                    size_t *nb_cols,
                    diet_matrix_order_t* order);

  /**
   * Get a container result from a dag
   */
  int
  getWfOutputContainer(diet_wf_desc_t* profile,
                       const char * id,
                       char** dataID);

  /**
   * Get sink results from a functional workflow
   * @return dataID contains the DAGDA ID of a container containing all results
   */
  int
  getWfSinkContainer(diet_wf_desc_t* profile,
                     const char * id,
                     char** dataID);

  /**
   * terminate a workflow session and free the memory
   *
   * @param profile profile of workflow to execute
   */
  void
  wf_free(diet_wf_desc_t * profile);

private:
  std::string name;
  /***************************************************************************/
  /*                           PRIVATE methods                               */
  /***************************************************************************/

  /**
   * Private constructor
   */
  CltWfMgr(const string& name);

  /**
   * Get current time (in milliseconds)
   */
  double
  getCurrTime();

  /**
   * Return the object IOR
   */
  /*const char *
    myIOR();*/
  const string& myName() const;


  /**
   * Return the DAG with a given identifier
   * @param dag_id Dag identifier
   * @return dag pointer or NULL if not found
   */
  Dag *
  getDag(std::string dag_id);

  /**
   * Initialize status for functional workflow
   */
  void
  initDagStatus(FWorkflow* wf);

  /**
   * Set the dag status regarding dags sent for a given functional workflow
   */
  void
  setWfSubmissionComplete(FWorkflow* wf);

  /**
   * Get the status regarding dags sent for a given functional workflow
   */
  bool
  isWfSubmissionComplete(FWorkflow* wf);

  /**
   * Common part of node execution
   */
  CORBA::Long
  execNodeCommon(const char * node_id,
                 const char * dag_id,
                 const char * sed,
                 const CORBA::ULong reqID,
                 corba_estimation_t& ev);

  /**
   * Common part of the dag submission
   * @param dagProfile  the diet profile containing the XML code for the Dag
   *                    (to be sent to MaDag) and eventually the wfReqId
   * @param dag a ref to a dag (may be NULL if parse is true)
   * @param parse will parse the XML if set to true
   * @param release will close the wfReqId on MaDag side if set to true
   */
  diet_error_t
  wfDagCallCommon(diet_wf_desc_t *dagProfile, Dag *dag, bool parse, bool release);


  /***************************************************************************/
  /*                          PRIVATE attributes                             */
  /***************************************************************************/

  /**
   * Unique instance reference
   */
  static CltWfMgr * myInstance;

  /**
   * Default data file name
   */
  static string defaultDataFileName;

  /**
   * MaDag CORBA object reference
   */
  MaDag_var myMaDag;

  /**
   * Master Agent reference
   */
  MasterAgent_var myMA;

#ifdef USE_LOG_SERVICE
  /**
   * Log service reference
   */
  DietLogComponent* myLC;
#endif

  /**
   * Wf Log Service Ref
   */
  WfLogService_var myLS;

  /**
   * Local workflow request ID counter
   * (different from wf request ID on MaDag)
   */
  int cltWfReqId;

  /**
   * Map for profiles and their dags or workflows
   */
  std::map<diet_wf_desc_t *, NodeSet *> myProfiles;

  /**
   * Map for metadags
   */
  std::map<CORBA::Long, MetaDag*> myMetaDags;

  /**
   * Dags status for functional workflows
   */
  std::map<FWorkflow*,bool> allDagsSent;

  /**
   * Dag sent counter
   */
  int dagSentCount;

  /**
   * Critical section
   */
  omni_mutex myLock;

  /**
   * Synchronisation semaphores
   */
  omni_semaphore mySem;

  bool instanciationPending;
  /**
   * Reference time
   */
  struct timeval refTime;
};

/* Forwarder part. */
class CltWfMgrFwdr : public POA_CltManFwdr,
                     public PortableServer::RefCountServantBase
{
private:
  Forwarder_ptr forwarder;
  char* objName;
public:
  CltWfMgrFwdr(Forwarder_ptr fwdr, const char* objName);

  virtual CORBA::Long execNodeOnSed(const char * node_id,
                                    const char * dag_id,
                                    const char * sed,
                                    const CORBA::ULong reqID,
                                    corba_estimation_t& ev);

  virtual CORBA::Long execNode(const char * node_id, const char * dag_id);

  virtual char * release(const char * dag_id, bool successful);

  virtual CORBA::Long ping();
};



#endif   /* not defined _CLTWFMGR._HH */

