/****************************************************************************/
/* DIET macros for use with thte asynchronous versions of dmat_manips.      */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Christophe PERA (Philippe.Combes@ens-lyon.fr)                       */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.3  2006/02/03 01:45:39  ecaron
 * Take into account the OS for omnithread thus MacOSX compilation is done.
 *
 * Revision 1.2  2003/06/30 11:15:12  cpera
 * Fix bugs in ReaderWriter and new internal debug macros.
 *
 * Revision 1.1  2003/06/16 17:12:49  pcombes
 * Move the examples using the asynchronous API into this directory.
 *
 ****************************************************************************/

#ifndef _GLOBAL_MACROS_HH_
#define _GLOBAL_MACROS_HH_

#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <iostream>
#include <omniconfig.h>
#include <omnithread.h>

using namespace std;
// Others debug Traces

#define traceTimer 1
#define traceThreadId 1
#define traceFile 1
#define traceLine 1
#define traceFunction 1

#define FILE_OUTPUT() \
	cout << "file=" << __FILE__ << "|";
#define LINE_OUTPUT() \
	cout << "line=" << __LINE__ << "|";
#define FUNCTION_OUTPUT() \
	cout << "function=" << __FUNCTION__ << "|";
#define OMNITHREADID_OUTPUT()\
	int id = omni_thread::self()->id(); \
	cout << "ThreaID=" << id << "|";
#define TIMER_OUTPUT() \
	timeval tval; \
	gettimeofday(&tval, NULL); \
	cout << "timer=" << tval.tv_sec << "," << tval.tv_usec << "|";
#define VARIABLE_OUTPUT(X) \
	cout << #X << "=" << X;
#define TEXT_OUTPUT(X) \
	printf X;
# if defined (DIET_NDEBUG)
#   define DIET_DEBUG(X)
# else
#   define DIET_DEBUG(X) DIET_TRACE_IMPL(X)
# endif

# define DIET_TRACE_IMPL(X) \
	do { \
		if (traceTimer == 1) { TIMER_OUTPUT() } \
		if (traceThreadId == 1){ OMNITHREADID_OUTPUT() } \
		if (traceFile == 1) { FILE_OUTPUT() } \
		if (traceLine == 1) { LINE_OUTPUT() } \
		if (traceFunction == 1) { FUNCTION_OUTPUT() } \
		X \
		cout << endl; \
		fflush(stdout); \
		fflush(stderr); \
  } while(0);

#endif /* _GLOBAL_MACROS_H_ */
