/***********************************************************/
/* Network statistics utility classes headers.             */
/*                                                         */
/*  Author(s):                                             */
/*    - Gael Le Mahec (lemahec@clermont.in2p3.fr)          */
/*                                                         */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/***********************************************************/
/* $Id$
 * $Log$
 * Revision 1.4  2008/11/07 14:32:14  bdepardo
 * Headers correction
 *
 *
 ***********************************************************/
#ifndef __NETWORKSTATS_HH__
#define __NETWORKSTATS_HH__

#include <map>
#include <string>

class NetworkStats {
protected:
  std::map<std::string, std::map<std::string, double> > values;
public:
  NetworkStats() : values() {}
  NetworkStats(const NetworkStats& stat) : values(stat.values) { }
  virtual ~NetworkStats() {}
  virtual double getStat(std::string src, std::string dest);
  virtual bool cmpStats(double s1, double s2) = 0;
  virtual void addStat(std::string src, std::string dest, double value) = 0;
};

class AvgNetworkStats : public NetworkStats {
protected:
  std::map<std::string, std::map<std::string, unsigned long> > nbStats;
public:
  AvgNetworkStats() : NetworkStats(), nbStats() {}
  AvgNetworkStats(const AvgNetworkStats& stat) : NetworkStats(stat), nbStats(stat.nbStats) {}
  virtual ~AvgNetworkStats() {}
  virtual double getStat(std::string src, std::string dest);
  virtual bool cmpStats(double s1, double s2);
  virtual void addStat(std::string src, std::string dest, double value);
};
#endif
