/****************************************************************************/
/* Hashed string object to be used with container                           */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Sylvain DAHAN (Sylvain.Dahan@lifc.univ-fcomte.fr)                   */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.3  2010/03/31 21:15:40  bdepardo
 * Changed C headers into C++ headers
 *
 * Revision 1.2  2004/10/08 12:04:02  hdail
 * Corrected mis-matched return values for hash in .cc and .hh.  Caused problem
 * for comptability with 64-bit machines.
 *
 * Revision 1.1  2004/09/29 13:35:32  sdahan
 * Add the Multi-MAs feature.
 *
 ****************************************************************************/

#include "ms_function.hh"
#include "KeyString.hh"
#include <cassert>
#include <cstring>

/* disp_state is a macro that displayed the attribut's value of the
   object and the name of the called methods */
//#include <cstdio>
//#define disp_state(fct) printf("KeyString::%s %s (%lx)\n", fct, (const char*)this->value ? (const char*)this->value : "empty-string", (long)(const char*)this->value)
#define disp_state(fct)

u_int32_t KeyString::hash(const char* id) {
  assert(id != NULL) ;
  u_int32_t result = 0 ;
  size_t length = strlen(id) ;
  for (size_t idIdx = 0 ; idIdx < length ; idIdx++)
    result = ((result<<1) | (result>>31))+ id[idIdx] ;
  return result ;
}


KeyString::KeyString(const char* value) {
  assert(value != NULL) ;
  disp_state("KeyString(const char* value)") ;
  this->str = ms_strdup(value) ;
  hashValue = hash(str) ;
  disp_state("--KeyString(const char* value)") ;
}


KeyString::KeyString(const KeyString & value) {
  disp_state("KeyString(const KeyString & value)") ;
  str = ms_strdup(value.str) ;
  hashValue = value.hashValue ;
  disp_state("--KeyString(const KeyString & value)") ;
}

KeyString::~KeyString() {
  disp_state("~KeyString()") ;
  ms_strfree(str) ;
}


KeyString &
KeyString::operator= (const KeyString & value) {
  disp_state("operator= (const KeyString & value)") ;
  ms_strfree(str) ;
  str = ms_strdup(value.str) ;
  hashValue = value.hashValue ;
  disp_state("--operator= (const KeyString & value)") ;
  return *this ;
}

KeyString::operator const char*() const {
  disp_state("KeyString::operator corba_ma_identifier_t()") ;
  return str ;
}


bool operator< (const KeyString & a, const KeyString & b) {
  bool result ;
  if (a.hashValue == b.hashValue)
    result = strcmp(a.str, b.str) < 0 ;
  else
    result = a.hashValue < b.hashValue ;
  return result ;
}


bool operator== (const KeyString & a, const KeyString & b) {
  return (a.hashValue == b.hashValue)
    && (strcmp(a.str, b.str) == 0) ;
}
