/****************************************************************************/
/* Plug-in scheduler: DIET-internal estimation vector access functions      */
/*                                                                          */
/* Author(s):                                                               */
/*    - Alan Su (Alan.Su@ens-lyon.fr)                                       */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.4  2010/01/14 13:15:08  bdepardo
 * "\n" -> endl
 *
 * Revision 1.3  2006/11/01 14:13:02  ecaron
 * Add missing header for release
 *
 * Revision 1.2 2005/09/07 07:42:26 hdail
 * Correction of small indexing bug.
 *
 * Revision 1.1 2005/08/31 15:01:10; alsu
 * New plugin scheduling interface: implementation of the DIET-internal
 * estimation vector access functions
 *
 ****************************************************************************/
#ifndef OLD_EV
#include "DIET_data.h"

#include "debug.hh"
#include "est_internal.hh"

#define EV_ELT(e,i) (((e)->estValues)[(i)])
#define EV_LEN(e) ((e)->estValues.length())
#define EV_SETLEN(e,l) ((e)->estValues.length(l))

int
diet_est_set_internal(estVector_t ev, int tag, double value)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", -1);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ << ": tag must be non-negative (" << tag << ")" << endl, -1);
  }

  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        (EV_ELT(ev, valIter)).v_idx < 0) {
      (EV_ELT(ev, valIter)).v_value = value;
      return (1);
    }
  }

  EV_SETLEN(ev, EV_LEN(ev) + 1);
  (EV_ELT(ev, EV_LEN(ev)-1)).v_tag = tag;
  (EV_ELT(ev, EV_LEN(ev)-1)).v_idx = EST_IDX_SCALAR;
  (EV_ELT(ev, EV_LEN(ev)-1)).v_value = value;
  
  return (1);
}

double
diet_est_get_internal(estVectorConst_t ev, int tag, double errVal)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", errVal);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ <<
          ": tag must be non-negative (" <<
          tag <<
          ")" << endl,
          errVal);
  }

  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        (EV_ELT(ev, valIter)).v_idx < 0) {
      return ((EV_ELT(ev, valIter)).v_value);
    }
  }

  return (errVal);
}

int
diet_est_defined_internal(estVectorConst_t ev, int tag)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", -1);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ << ": tag must be non-negative (" << tag << ")" << endl, -1);
  }

//   cout << "AS: [" << __FUNCTION__ << "] num values = " << EV_LEN(ev) << endl;

  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        (EV_ELT(ev, valIter)).v_idx < 0) {
      return (1);
    }
  }

  return (0);
}

int
diet_est_array_size_internal(estVectorConst_t ev, int tag)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", -1);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ << ": tag must be non-negative (" << tag << ")" << endl, -1);
  }

  int maxIdx = -1;
  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        maxIdx < (EV_ELT(ev, valIter)).v_idx) {
      maxIdx = (EV_ELT(ev, valIter)).v_idx;
    }
  }

  return (maxIdx + 1);
}

int
diet_est_array_set_internal(estVector_t ev, int tag, int idx, double value)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", -1);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ << ": tag must be non-negative (" << tag << ")" << endl, -1);
  }
  if (idx < 0) {
    ERROR(__FUNCTION__ << ": idx must be non-negative (" << idx << ")" << endl, -1);
  }

  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        (EV_ELT(ev, valIter)).v_idx == idx) {
      (EV_ELT(ev, valIter)).v_value = value;
      return (1);
    }
  }

  EV_SETLEN(ev, EV_LEN(ev) + 1);
  (EV_ELT(ev, EV_LEN(ev)-1)).v_tag = tag;
  (EV_ELT(ev, EV_LEN(ev)-1)).v_idx = idx;
  (EV_ELT(ev, EV_LEN(ev)-1)).v_value = value;

  return (1);
}

double
diet_est_array_get_internal(estVectorConst_t ev,
                            int tag,
                            int idx,
                            double errVal)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", errVal);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ <<
          ": tag must be non-negative (" <<
          tag <<
          ")" << endl,
          errVal);
  }
  if (idx < 0) {
    ERROR(__FUNCTION__ <<
          ": idx must be non-negative (" <<
          idx <<
          ")" << endl,
          errVal);
  }

  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        (EV_ELT(ev, valIter)).v_idx == idx) {
      return ((EV_ELT(ev, valIter)).v_value);
    }
  }

  return (errVal);
}

int
diet_est_array_defined_internal(estVectorConst_t ev, int tag, int idx)
{
  if (ev == NULL) {
    ERROR(__FUNCTION__ << ": NULL estimation vector", -1);
  }
  if (tag < 0) {
    ERROR(__FUNCTION__ << ": tag must be non-negative (" << tag << ")" << endl, -1);
  }
  if (idx < 0) {
    ERROR(__FUNCTION__ << ": idx must be non-negative (" << idx << ")" << endl, -1);
  }

  for (unsigned int valIter = 0 ; valIter < EV_LEN(ev) ; valIter++) {
    if (tag == (EV_ELT(ev, valIter)).v_tag &&
        (EV_ELT(ev, valIter)).v_idx == idx) {
      return (1);
    }
  }

  return (0);
}
#endif /* OLD_EV */
