/****************************************************************************/
/* DIET Fault Detector Application watcher                                  */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Aurlien Bouteiller (aurelien.bouteiller@inria.fr)                  */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$ 
 * $Log$
 * Revision 1.5  2010/10/05 03:15:39  bdepardo
 * Infinit loop rewriting
 *
 * Revision 1.4  2010/03/31 21:15:40  bdepardo
 * Changed C headers into C++ headers
 *
 * Revision 1.3  2006/06/20 13:29:16  abouteil
 *
 *
 * Chandra&Toueg&Aguilera fault detector implementation.
 *
 * A
 *
 ****************************************************************************/	
#include <sys/time.h>
#include <sys/msg.h>

#include <cerrno>
#include "threadoverlay.h"

#include "fdd.h"
#include "formulae.h"


void* setup_application_watcher (void *nothing)
{
  watched_process p;
  struct timeval t;
  thread_mutex_lock (&watched_mutex);
  for(;;) {
    if (heap_is_empty (watched)) {
      /* wait for a watched process to be added */
      thread_cond_wait (&new_watched_cond, &watched_mutex);
    } else {
      /* wait for either a new watched process to be added,
       * or the current next process to need attention
       */
      p = (watched_process) heap_peek (watched);
      thread_cond_timedwait (&new_watched_cond,
        &watched_mutex, &p->tau_lp1);
    }
    /* an application may have become suspect */
    p = (watched_process) heap_extract (watched);

    p->tau_lp1.tv_sec = p->next_tau_lp1.tv_sec;
    p->tau_lp1.tv_nsec = p->next_tau_lp1.tv_nsec;
    gettimeofday (&t, NULL);

    if (timespec_to_seconds (&p->tau_lp1) < timeval_to_seconds (&t)) {
      /* suspect !*/       
      p->suspect = 1;
      fd_notify(p->handle);
    } else {
      /* not suspect */
      heap_insert (watched, p);
    }
  }
}

