/****************************************************************************/
/* DIET Fault Detector Client API headers                                   */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Aurlien Bouteiller (aurelien.bouteiller@inria.fr)                  */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$ 
 * $Log$
 * Revision 1.4  2010/12/17 09:48:01  kcoulomb
 * * Set diet to use the new log with forwarders
 * * Fix a CoRI problem
 * * Add library version remove DTM flag from ccmake because deprecated
 *
 * Revision 1.3  2006/06/20 13:29:16  abouteil
 *
 *
 * Chandra&Toueg&Aguilera fault detector implementation.
 *
 * A
 *
 ****************************************************************************/
#ifndef __FD_H__
#define __FD_H__

#include <unistd.h>
#include <netdb.h>

#ifdef USE_LOG_SERVICE
#include "DietLogComponent.hh"
#endif

/* type of handles - should be condidered opaque */
typedef struct _fd_handle {
	int suspect;	/* is the watched application suspect ? */
	double tdu;	/* QoS : upper bound on detection time */
	double tmrl;	/* QoS : lower bound on avg mistake recurrence time */
	double tmu;	/* QoS : upper bound on avg mistake duration */
	char* machine_name;	/* observed machine name */
	in_addr_t address;	/* observed machine IP address */
	int service_number;	/* observed service */
#ifdef USE_LOG_SERVICE
        DietLogComponent *log;  /* Diet centralized log service to use */
#endif
} _fd_handle;
typedef struct _fd_handle* fd_handle;

      /**************************\
       * the CLIENT library API *
      \**************************/

/* makes a new handle with all data set to default values */
fd_handle fd_make_handle (void);

/* gets the QoS characteristics of a handle */
void fd_get_qos (fd_handle handle, double *, double *, double *);

/* sets the QoS characteristics of a handle */
void fd_set_qos (fd_handle handle, double, double, double);

/* gets the machine name and service number of a handle */
void fd_get_service (fd_handle handle, char*, int, int*);

/* sets the machine name and service number of a handle */
int fd_set_service (fd_handle handle, const char*, int);

#ifdef USE_LOG_SERVICE
/* sets the diet logger of a handle */
void fd_set_logger(fd_handle handle, DietLogComponent *l);

/* gets the current diet logger used by a handle */
void fd_get_logger(fd_handle handle, DietLogComponent **l);
#endif

/* frees a handle */
void fd_free_handle (fd_handle handle);

/* observation */
int fd_observe (fd_handle handle);

/* liveliness checking */
int fd_alive (fd_handle handle);

/* warning specification */
void fd_warn (int signal);

/* transition handler setting */
void fd_set_transition_handler (void (*f) (fd_handle));

      /***************************\
       * the SERVICE library API *
      \***************************/
      
/* service registration */
void fd_register_service (pid_t client, int service_number);

/* service unregistration */
void fd_unregister_service (pid_t client, int service_number);

#endif /* __FD_H_ */
