/****************************************************************************/
/* DIET Fault Detector simple hashtable                                     */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Aurlien Bouteiller (aurelien.bouteiller@inria.fr)                  */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$ 
 * $Log$
 * Revision 1.6  2010/10/04 08:17:23  bdepardo
 * Changed memory management from C to C++ (malloc/free -> new/delete)
 * This corrects a bug at initialization
 *
 * Revision 1.5  2010/03/31 21:15:40  bdepardo
 * Changed C headers into C++ headers
 *
 * Revision 1.4  2006/11/01 14:02:25  ecaron
 * Correct wrong end line character (Ctrl+M)
 *
 * Revision 1.3  2006/06/20 13:29:16  abouteil
 *
 *
 * Chandra&Toueg&Aguilera fault detector implementation.
 *
 * A
 *
 ****************************************************************************/
#include <cstdlib>

#include "hashtable.h"


/* constructor */
hashtable hashtable_new (unsigned int (*hash)(void *),
	unsigned int (*matches)(void *, void*))
{
	int i;
	hashtable result = new _hashtable;
	result->hash = hash;
	result->matches = matches;
	result->data = (list*) malloc (HASHTABLE_SIZE * sizeof (list));
	for (i = 0; i < HASHTABLE_SIZE; ++i)
		result->data[i] = EMPTY_LIST;
	return result;
}

/* destructor (does NOT free the contained cells) */
void hashtable_destroy (hashtable h)
{
	int i;
	for (i = 0; i < HASHTABLE_SIZE; ++i)
		list_destroy (h->data[i]);
	delete h;
}


/* insertion */
void hashtable_insert (hashtable h, void* c)
{
	int key = (h->hash (c)) % HASHTABLE_SIZE;
	h->data[key] = list_cons (h->data[key], c);
}

/* search */
void* hashtable_search (hashtable h, void* c)
{
	void * e;
	list l = h->data [(h->hash (c)) % HASHTABLE_SIZE];
	while (l != NULL) {
		e = list_head (l);
		if (h->matches (c, e)) {
			return e;
		} else {
			l = list_tail (l);
		}
	}
	return NULL;
}

/* removing */
void hashtable_remove (hashtable h, void* c)
{
	list previous, current;
	int key = (h->hash (c)) % HASHTABLE_SIZE;
	list l = h->data [key];
	
	/* special case 1 : empty list */
	if (l == NULL) return;

	/* special case 2 : searched element is at head */
	current = list_tail (l);
	if (h->matches (list_head (l), c)) {
		delete l;
		h->data [key] = current;
		return;
	}

	/* normal case */
	previous = l;
	while (current != NULL) {
		if (h->matches (list_head (current), c)) {
			previous->tail = list_tail (current);
			delete current;
			return;
		} else {
			previous = current;
			current = list_tail (current);
		}
	}
}

