/****************************************************************************/
/* DIET Fault Detector simple hashtable headers                             */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Aurlien Bouteiller (aurelien.bouteiller@inria.fr)                  */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$ 
 * $Log$
 * Revision 1.4  2006/11/01 14:02:25  ecaron
 * Correct wrong end line character (Ctrl+M)
 *
 * Revision 1.3  2006/06/20 13:29:16  abouteil
 *
 *
 * Chandra&Toueg&Aguilera fault detector implementation.
 *
 * A
 *
 ****************************************************************************/
#ifndef _HASHTABLE_H_
#define _HASHTABLE_H_


#include "list.h"


/* some prime number */
#define HASHTABLE_SIZE 211


/* hash table = array of lists with user-supplied functions :
 * - hash : takes an element and returns an integer in [0..MAXINT]
 * - matches : tests equality between searched and candidate
 */
typedef struct _hashtable {
	unsigned int (*hash) (void *);
	unsigned int (*matches) (void*, void*);	/* true means match */
	list* data;
} _hashtable;
typedef struct _hashtable * hashtable;

/* constructor and destructor */
hashtable hashtable_new (unsigned int (*hash)(void *),
	unsigned int (*compare)(void *, void*));
void hashtable_destroy (hashtable);


/* insertion, search, removing */
void hashtable_insert (hashtable, void*);
void* hashtable_search (hashtable, void*);
void hashtable_remove (hashtable, void*);


#endif /* HASHTABLE_H_ */

