/****************************************************************************/
/* DIET Fault Detector simple heap (decr. ordering)                         */
/*                                                                          */
/*  Author(s):                                                              */
/*    - Aurlien Bouteiller (aurelien.bouteiller@inria.fr)                  */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$ 
 * $Log$
 * Revision 1.5  2010/10/04 08:17:23  bdepardo
 * Changed memory management from C to C++ (malloc/free -> new/delete)
 * This corrects a bug at initialization
 *
 * Revision 1.4  2010/03/31 21:15:40  bdepardo
 * Changed C headers into C++ headers
 *
 * Revision 1.3  2006/06/20 13:29:16  abouteil
 *
 *
 * Chandra&Toueg&Aguilera fault detector implementation.
 *
 * A
 *
 ****************************************************************************/
#include <cstdlib>
#include "heap.h"

/* minimum size */
#define HEAP_MIN_SIZE 1024


/* constructor */
heap heap_new (int (*compare)(void*, void*))
{
	heap result = new _heap;
	result->compare = compare;
	result->data = (void **) malloc (HEAP_MIN_SIZE * sizeof (void *));
	result->nb_elements = 0;
	result->size = HEAP_MIN_SIZE;
	return result;
}

/* destructor */
void heap_destroy (heap h)
{
	free (h->data);
	delete h;
}

/* emptiness test */
int heap_is_empty (heap h)
{
	return (h->nb_elements == 0);
}

/* utility functions */

/* memory management */
void heap_check_more (heap h)
{
	if (h->nb_elements > h->size) {
		h->size = 2*h->size;
		h->data = (void **) realloc (h->data, h->size * sizeof (void*));
	}
}

void heap_check_less (heap h)
{
	if ((h->nb_elements < h->size / 3) && (h->size > HEAP_MIN_SIZE)) {
		h->size = h->size / 2;
		h->data = (void **) realloc (h->data, h->size * sizeof (void*));
	}
}

/* heap navigation */
int heap_parent (int i)
{
	return (i-1)/2;
}

int heap_left (int i)
{
	return (i*2)+1;
}

int heap_right (int i)
{
	return (i*2)+2;
}

/* insertion */
void heap_insert (heap h, void* c)
{
	void* tmp;
	int p;
	int i;
	
	i = h->nb_elements;
	++h->nb_elements;
	heap_check_more (h);
	h->data[i] = c;
	p = heap_parent (i);
	/* reorder elements in heap to keep it partially sorted */
	while ((i > 0) && (h->compare (h->data[i], h->data[p]) < 0)) {
		tmp = h->data[i];
		h->data[i] = h->data[p];
		h->data[p] = tmp;
		i = p;
		p = heap_parent (i);
	}
}

/* peek */
void* heap_peek (heap h)
{
	if (h->nb_elements == 0)
		return NULL;
	else
		return h->data[0];
}

/* extraction */
void* heap_extract (heap h)
{
	int i, r, l, m;
	void* tmp;
	void* result;
	
	if (h->nb_elements == 0) return NULL;

	result = h->data[0];

	h->data[0] = h->data[h->nb_elements-1];
	
	--h->nb_elements;
	heap_check_less (h);

	/* reordering */
	i = 0;
	while (1) {
		l = heap_left (i);
		r = heap_right (i);

		if ((l < h->nb_elements) &&
			(h->compare (h->data[l], h->data[i]) < 0)) {
			m = l;
		} else {
			m = i;
		}
		if ((r < h->nb_elements) &&
			(h->compare (h->data[r], h->data[m]) < 0)) {
			m = r;
		}
		if (m == i) {
			break;
		} else {
			tmp = h->data[i];
			h->data[i] = h->data[m];
			h->data[m] = tmp;
			i = m;
		}
	};
	return result;
}

