/****************************************************************************/
/* CoRI Collectors of ressource information (Memory part)                   */
/*                                                                          */
/* Author(s):                                                               */
/*    - Frauenkron Peter (Peter.Frauenkron@ens-lyon.fr)                     */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.6  2011/03/31 17:45:29  hguemar
 * more robust data input: add field width limits to scanf/fscanf
 *
 * Revision 1.5  2011/02/01 22:46:58  bdepardo
 * Removed useless function: test_filename, it wasn't even part of Easy_Memory
 * and never used throughout the entire project.
 *
 * Revision 1.4  2011/01/21 00:00:53  bdepardo
 * Removed memory leak.
 * Removed: llocation with fopen, while doesn't release it.
 *
 * Revision 1.3  2010/12/17 09:48:02  kcoulomb
 * * Set diet to use the new log with forwarders
 * * Fix a CoRI problem
 * * Add library version remove DTM flag from ccmake because deprecated
 *
 * Revision 1.2  2010/03/31 21:15:41  bdepardo
 * Changed C headers into C++ headers
 *
 * Revision 1.1  2010/03/03 14:26:35  bdepardo
 * BEWARE!!!
 * Huge modifications to take into account CYGWIN.
 * Lots of files' directory have been changed.
 *
 * Revision 1.8  2006/12/06 22:14:25  ecaron
 * Header bug fix
 *
 * Revision 1.7  2006/11/28 13:30:31  ecaron
 * Add $Log: from cvs log
 *
 *
 * Revision 1.3 2006/01/15 15:37:07 pfrauenk
 * CoRI: debug.hh is used now, cleanup, minor changes
 *
 * Revision 1.2 2005/12/20 21:48:46 pfrauenk
 * CoRI defined symbols renamed - bug fixes thx to Raphael Bolze
 *
 * Revision 1.1 2005/12/15 10:12:17 pfrauenk
 * CoRI functionality added by Peter Frauenkron
 ****************************************************************************/
#include <unistd.h> //int getpagesize (void)
#ifdef CORI_HAVE_SYS_SYSINFO
#include <sys/sysinfo.h> // get_phys_pages (),get_avphys_pages ()
#endif
#include <cstdio>
#include "Cori_Easy_Memory.hh"
#include <cstdlib> //system()
#include <iostream>
#include <fstream>
#include <cstring>
//#include <string.h>//srtcmp
#include <string>
#include "debug.hh"

using namespace std;

int
Easy_Memory::get_Total_Memory(double * result)
{
  double temp1,temp2,temp3;
  fprintf (stderr, "Getting total memory \n");
  if (!get_Info_Memory_byProcMem(&temp1,0)){
    fprintf (stderr, "Getting total memory 1 : result = %lf\n", temp1);
    *result=temp1;
    return 0;
  }
  else
  if (!get_Total_Memory_bysysinfo(&temp1)
      &&(!get_Avail_Memory_byvmstat(&temp2))
      &&(!get_Avail_Memory_bysysinfo(&temp3)))
  {
    fprintf (stderr, "Getting total memory 2 : result1 = %lf, result2 = %lf, result3=%lf\n",temp1, temp2, temp3);
    double pagesize= temp2/temp3;
    *result=temp1*pagesize;
    return 0;
  }
  else{
      *result=0;
      return 1;
    }
}
int
Easy_Memory::get_Avail_Memory(double * result)
{
  double temp;
   if (!get_Info_Memory_byProcMem(&temp,1)){
    *result=temp;
    return 0;
  }
  else
  if (!get_Avail_Memory_byvmstat(&temp)){
    *result=temp;
    return 0;
  }
  else{
      *result=0;
      return 1;
    }
}

/****************************************************************************/
/* Private methods                                                          */
/****************************************************************************/

int
Easy_Memory::get_Total_Memory_bysysinfo(double * result)
{
  // using  get_phys_pages ()
  // returns the total number of physical pages the system has
  // and
  // int getpagesize (void)
  // Inquire about the virtual memory page size of the machine.
#if defined CORI_HAVE_get_avphys_pages && defined (CORI_HAVE_getpagesize)
  fprintf (stderr, "phys %lf \n", (double)get_phys_pages ());
  fprintf (stderr, "page %lf \n", (double)getpagesize ());
  *result=(get_phys_pages ()/1024)*(getpagesize()/1024);
  return 0;
#else
    //chercher autre solutions
    return 1;
#endif
}

int
Easy_Memory::get_Avail_Memory_bysysinfo(double * result)
{
  // using get_avphys_pages ()
  // returns the number of physical available pages the system has
  // and
  // int getpagesize (void)
  // Inquire about the virtual memory page size of the machine.
#if defined CORI_HAVE_get_avphys_pages && defined (CORI_HAVE_getpagesize)
  *result=get_avphys_pages ()*getpagesize()/(1024*1024);
  return  0;
#else
  //chercher autres sol
  return 1;
#endif
}

int
Easy_Memory::get_Avail_Memory_byvmstat(double * result)
{
  int returnval=1;
  FILE * myfile =popen("vmstat","r");
   char word[256];
    if ((myfile!=NULL)){
      if (!feof(myfile)){
	fgets(word, 256, myfile);
	  if (!feof(myfile)){
	    fgets(word, 256, myfile);
	    int i=0;
	    while ((!feof(myfile))&&(i<4)){
	      fscanf (myfile, "%255s", word);
	      i++;
	    }
	    if (i==4){
	      *result = atof ( word )/1024;
	      returnval=0;
	    }
	    else return 1;
	  }
      }
      pclose(myfile);
  }
  return returnval;
}


/*
 cat /proc/meminfo*/
int
Easy_Memory::get_Info_Memory_byProcMem(double* resultat,
				       int freemem)
{
#ifdef CORI_HAVE_PROCMEM

  	char  word[256];
	char demanded[256];
	/* looking in the /proc/meminfo data file*/
	ifstream file ("/proc/meminfo");

	if (freemem)
	  strcpy (demanded,"MemFree:");
	else strcpy (demanded,"MemTotal:");


	if (file.is_open())
	{
	  while ( ! file.eof() )  //look at the whole file
	  {
	    if ( strcmp (demanded , word ) == 0 )
	    {
		file >> word;
		*resultat=atof(word)/1024;
		return 0;
	    }
	    file >> word;
	  }
	  file.close();
	}
  	else{
	  TRACE_TEXT(TRACE_MAX_VALUE,"Error on reading file");
	  return 1;
	}
#endif //CORI_HAVE_PROCMEM
 return 1;

}


// int main (){
//   Easy_Memory mem;
//   double maxmem;
//   double availmem;
//   if (!mem.get_Total_Memory(&maxmem))
//      printf("%d\n", maxmem);
//   else printf("no info\n");

//   if (!mem.get_Avail_Memory(&availmem))
//       printf("%d\n", availmem);
//   else printf("no info\n");



// }
