/****************************************************************************/
/* Dag description                                                          */
/*                                                                          */
/* Author(s):                                                               */
/* - Abdelkader AMAR (Abdelkader.Amar@ens-lyon.fr)                          */
/* - Benjamin Isnard (benjamin.isnard@ens-lyon.fr)                          */
/*                                                                          */
/*  This file is part of DIET .                                             */
/*                                                                          */
/*  Copyright ENS Lyon, INRIA, UCBL, SysFera (2000)                         */
/*                                                                          */
/*  - Frederic.Desprez@ens-lyon.fr (Project Manager)                        */
/*  - Eddy.Caron@ens-lyon.fr (Technical Manager)                            */
/*  - Tech@sysfera.com (Maintainer and Technical Support)                   */
/*                                                                          */
/*  This software is a computer program whose purpose is to provide an      */
/*  easy and transparent access to distributed and heterogeneous            */
/*  platforms.                                                              */
/*                                                                          */
/*                                                                          */
/*  This software is governed by the CeCILL license under French law and    */
/*  abiding by the rules of distribution of free software.  You can  use,   */
/*  modify and/ or redistribute the software under the terms of the CeCILL  */
/*  license as circulated by CEA, CNRS and INRIA at the following URL       */
/*  "http://www.cecill.info".                                               */
/*                                                                          */
/*  As a counterpart to the access to the source code and  rights to copy,  */
/*  modify and redistribute granted by the license, users are provided      */
/*  only with a limited warranty  and the software's author,  the holder    */
/*  of the economic rights,  and the successive licensors  have only        */
/*  limited liability.                                                      */
/*                                                                          */
/*  In this respect, the user's attention is drawn to the risks             */
/*  associated with loading,  using,  modifying and/or developing or        */
/*  reproducing the software by the user in light of its specific status    */
/*  of free software, that may mean  that it is complicated to              */
/*  manipulate, and  that  also therefore means  that it is reserved for    */
/*  developers and experienced professionals having in-depth computer       */
/*  knowledge. Users are therefore encouraged to load and test the          */
/*  software's suitability as regards their requirements in conditions      */
/*  enabling the security of their systems and/or data to be ensured and,   */
/*  more generally, to use and operate it in the same conditions as         */
/*  regards security.                                                       */
/*                                                                          */
/*  The fact that you are presently reading this means that you have had    */
/*  knowledge of the CeCILL license and that you accept its terms.          */
/*                                                                          */
/****************************************************************************/
/* $Id$
 * $Log$
 * Revision 1.29  2010/07/20 09:20:11  bisnard
 * integration with eclipse gui and with dietForwarder
 *
 * Revision 1.28  2010/03/31 21:15:41  bdepardo
 * Changed C headers into C++ headers
 *
 * Revision 1.27  2010/03/08 13:50:48  bisnard
 * handle node ready event (for logging)
 *
 * Revision 1.26  2009/09/25 12:49:11  bisnard
 * avoid deadlocks due to new thread mgmt in DagNodeLauncher
 *
 * Revision 1.25  2009/08/26 10:33:08  bisnard
 * implementation of workflow status & restart
 *
 * Revision 1.24  2009/07/07 09:02:00  bisnard
 * modified toXML method
 *
 * Revision 1.23  2009/05/15 11:02:55  bisnard
 * added makespan calculation
 *
 * Revision 1.22  2009/04/17 08:54:43  bisnard
 * renamed Node class as WfNode
 *
 * Revision 1.21  2009/02/06 14:55:08  bisnard
 * setup exceptions
 *
 * Revision 1.20  2009/01/22 09:01:09  bisnard
 * added client method to retrieve workflow container output
 *
 * Revision 1.19  2009/01/16 13:55:36  bisnard
 * changes in dag event handling methods
 *
 * Revision 1.18  2008/12/09 12:12:14  bisnard
 * added reference to FWorkflow
 *
 * Revision 1.17  2008/12/02 10:11:41  bisnard
 * modified node precedence check
 *
 * Revision 1.16  2008/10/22 09:29:00  bisnard
 * replaced uint by standard type
 *
 * Revision 1.15  2008/10/20 08:01:12  bisnard
 * removed createNode method from NodeSet class
 *
 * Revision 1.14  2008/10/14 13:31:01  bisnard
 * new class structure for dags (DagNode,DagNodePort)
 *
 * Revision 1.13  2008/09/30 15:32:53  bisnard
 * - using simple port id instead of composite ones
 * - dag nodes linking refactoring
 * - prevNodes and nextNodes data structures modified
 * - prevNodes initialization by Node::setNodePredecessors
 *
 * Revision 1.12  2008/09/19 13:11:07  bisnard
 * - added support for containers split/merge in workflows
 * - added support for multiple port references
 * - profile for node execution initialized by port (instead of node)
 * - ports linking managed by ports (instead of dag)
 *
 * Revision 1.11  2008/07/17 12:19:18  bisnard
 * Added dag cancellation method
 *
 * Revision 1.10  2008/07/11 07:56:01  bisnard
 * provide list of failed nodes in case of cancelled dag
 *
 * Revision 1.9  2008/06/25 10:07:12  bisnard
 * - removed debug messages
 * - Node index in wf_response stored in Node class (new attribute submitIndex)
 *
 * Revision 1.8  2008/06/19 10:17:13  bisnard
 * new attribute to store DAG start time
 *
 * Revision 1.7  2008/06/02 08:35:39  bisnard
 * Avoid MaDag crash in case of client-SeD comm failure
 *
 * Revision 1.6  2008/06/01 09:18:43  rbolze
 * remove myreqID attribute from the RunnableNode
 * add getReqID() method which return the reqID stored in the diet_profile_t
 * add 2 methods: showDietReqID and getAllDietReqID
 * this is use to send feedback through logservice
 *
 * Revision 1.5  2008/05/16 12:32:10  bisnard
 * API function to retrieve all workflow results
 *
 * Revision 1.4  2008/04/28 12:15:00  bisnard
 * new NodeQueue implementation for FOFT
 * nodes sorting done by Dag instead of scheduler
 * method to propagate delay at execution (FOFT)
 *
 * Revision 1.3  2008/04/21 14:36:59  bisnard
 * use nodeQueue to manage multiwf scheduling
 * use wf request identifer instead of dagid to reference client
 * renamed WfParser as DagWfParser
 *
 * Revision 1.2  2008/04/14 09:10:40  bisnard
 *  - Workflow rescheduling (CltReoMan) no longer used with MaDag v2
 *  - AbstractWfSched and derived classes no longer used with MaDag v2
 *
 * Revision 1.1  2008/04/10 08:38:50  bisnard
 * New version of the MaDag where workflow node execution is triggered by the MaDag agent and done by a new CORBA object CltWfMgr located in the client
 *
 ****************************************************************************/

#ifndef _DAG_HH_
#define _DAG_HH_

#include <string>
#include <vector>
#include <map>
#include <list>
#include <iostream>

#include "DIET_client.h" // for diet_ReqID_t

#include <sys/time.h>
#include <ctime>

#include "WfNode.hh"
#include "MasterAgent.hh"
#include "NodeSet.hh"

using namespace std;

class FWorkflow;
class DagNode;
class DagNodeOutPort;
class DagScheduler;

/*****************************************************************************/
/*                              CLASS Dag                                    */
/*****************************************************************************/

class Dag : public NodeSet {
public:
  /*********************************************************************/
  /* constructors/destructor                                           */
  /*********************************************************************/

  /**
   * Dag constructor WITHOUT DAG execution agent
   * @param id	required non-empty identifier
   */
  Dag(string id);

  /**
   * Dag constructor WITH DAG execution agent (DIET MasterAgent)
   * @param id  required non-empty identifier
   * @param MA	master agent (DIET platform MA used for dag execution)
   */
  Dag(string id, MasterAgent_var& MA);

  /**
   * Dag destructor
   */
  virtual ~Dag();

  /***************************************************/
  /*            NodeSet methods                      */
  /***************************************************/

  /**
   * Get the node with given identifier
   *
   * @param nodeId node identifier
   * @return pointer to node (does not return NULL)
   */
  virtual WfNode *
  getNode(const string& nodeId) throw (WfStructException);

  /**
   * check the precedence between node
   * this function check only the precedence between nodes, it doesn't
   * link the ports
   */
  virtual void
  checkPrec(NodeSet* contextNodeSet) throw (WfStructException);
  
  /***************************************************/
  /* event message types                             */
  /***************************************************/
  
  enum eventMsg_e {
    MODID,
    EMPTY,
    STATE
  };

  /***************************************************/
  /*               public methods                    */
  /***************************************************/

  /**
   * Set the dag id
   */
  void
  setId(const string& id);

  /**
   * Get the dag id
   */
  const string&
  getId() const;

  /**
   * Set the functional wf that created this dag
   */
  void
  setWorkflow(FWorkflow * wf);

  /**
   * Get the functional wf
   */
  FWorkflow *
  getWorkflow() const throw (WfStructException);

  /**
   * Set the execution agent
   */
  void
  setExecutionAgent(MasterAgent_var& MA);

  /**
   * Get the execution agent
   */
  MasterAgent_var&
  getExecutionAgent();

  /**
   * Create a new node of the dag
   * (allocates a new node and insert it in the dag)
   * @param id      node id (not the complete id)
   * @param wf      workflow ref (optional)
   * @return pointer to the node (does not return NULL)
   */
  DagNode*
  createDagNode(const string& id, FWorkflow* wf = NULL) throw (WfStructException);

  /**
   * return a dag's node
   * @return pointer to the node (does not return NULL)
   */
  DagNode *
  getDagNode(const string& nodeId) throw (WfStructException);

  /**
   * remove a node from the dag
   */
  void
  removeNode(const string& nodeId) throw (WfStructException);

  /**
   * return the size of the Dag (the nodes number and not the dag length)
   */
  unsigned int
  size();

  /**
   * return an iterator on the first node
   * (according to the map and not to the dag structure)
   */
  map <string, DagNode *>::iterator
  begin();

  /**
   * return an iterator on the last node *
   * (according to the map and not to the dag structure) *
   */
  map <string, DagNode *>::iterator
  end();
  
  /**
   * Returns a string description of the dag
   */
  string
  toString() const;

  /**
   * returns the XML description of the dag
   */
  void
  toXML(ostream& output) const;

  /**
   * link all ports of the dag
   */
  void
  linkAllPorts() throw (WfStructException);

  /**
   * check if the dag execution is ongoing
   */
  bool
  isRunning();

  /**
   * check if the dag execution is completed *
   */
  bool
  isDone();

  /**
   * check if the dag execution is cancelled *
   */
  bool
  isCancelled();

  /**
   * get a scalar result of the workflow
   *
   * @param id    the port identifier (complete id 'node id'#'port id')
   * @param value the data to store the returned scalar value
   */
  int
  get_scalar_output(const char * id, void** value);

  /**
   * get a string result of the workflow
   *
   * @param id    the port identifier
   * @param value the data to store the returned string value
   */
  int
  get_string_output(const char * id, char** value);


  /**
   * Get a file result of the workflow
   *
   * @param id    the output port id (prefixed with the node id)
   * @param size  the size of the returned file
   * @param path  the file name
   */
  int
  get_file_output (const char * id,
		   size_t* size, char** path);

  /**
   * Get a matrix result of the workflow
   *
   * @param id      the output port id (prefixed with the node id)
   * @param value   the matrix reference
   * @param nb_rows the matrix rows count
   * @param nb_cols the matrix columns count
   * @param order   the matrix order
   */
  int
  get_matrix_output (const char * id, void** value,
		     size_t* nb_rows, size_t *nb_cols,
		     diet_matrix_order_t* order);

  /**
   * Get a container result of the workflow
   * @param id      the output port id (prefixed with the node id)
   * @param dataID  a ref to a string that will contain a copy of the container dataID
   */
  int
  get_container_output (const char * id, char** dataID);

  /**
   * Get all the results and display them. This function doesn't returned
   * the value.
   */
  void
  displayAllResults(ostream& output);

  /**
   * Delete all results of the workflow (includes intermediary and final
   * results)
   */
  void
  deleteAllResults();

  /**
   * Get all dag nodes sorted by priority
   *
   * @return ref to a vector of nodes (to be deleted by caller)
   */
  vector<DagNode*>&
  getNodesByPriority();

  /**
   * Get the input nodes
   */
  vector<DagNode*>
  getInputNodes();

  /**
   * Set all input nodes as ready
   * (used to notify node queue)
   */
  void
  setInputNodesReady(DagScheduler* scheduler);

  /**
   * get the estimated makespan of the DAG
   * @deprecated
   */
//   double
//   getEstMakespan();

  /**
   * get the estimated earliest finish time of the DAG
   */
  double
  getEFT();

  /**
   * set the start time of the DAG
   */
  void
  setStartTime(double time);

  /**
   * get the start time of the DAG
   */
  double
  getStartTime();

  /**
   * set the finish time of the DAG
   */
  void
  setFinishTime(double time);

  /**
   * get the makespan of the DAG (finish - start)
   */
  double
  getMakespan();

  /**
   * get the estimated delay of the DAG
   */
  double
  getEstDelay();

  /**
   * set the estimated delay of the DAG
   * (updated by an exit node)
   */
  void
  setEstDelay(double delay);

  /**
   * recursively updates the estimated delay starting from a node
   * and applying it to successors
   */
  bool
  updateDelayRec(DagNode * node, double newDelay);

  /**
   * Notify dag of node end of execution (when successful)
   */
  void
  setNodeDone(DagNode* node, DagScheduler* scheduler = NULL);

  /**
   * notify dag of node execution failure
   */
  void
  setNodeFailure(string nodeId, DagScheduler* scheduler = NULL);

  /**
   * get the list of failed nodes
   */
  const list<string>&
  getNodeFailureList();

  /**
   * set the dag as cancelled
   */
  void
  setAsCancelled(DagScheduler* scheduler = NULL);

  void
  showDietReqID();

  vector<diet_reqID_t>
  getAllDietReqID();

private:

  /*********************************************************************/
  /* private fields                                                    */
  /*********************************************************************/

  /**
   * The dag id
   */
  string myId;

  /**
   * Workflow for which dag is an instance
   */
  FWorkflow * myWf;

  /**
   * Ref to DAG execution agent
   */
  MasterAgent_var  myExecAgent;

  /**
   * Dag nodes *
   */
  map<string, DagNode *> nodes;

  /**
   * start time
   */
  double startTime;

  /**
   * finish time
   */
  double finishTime;

  /**
   * estimated delay
   */
  double estDelay;

  /**
   * Failed nodes list
   */
  list<string> failedNodes;

  /**
   * Temporary dag flag. Used to not delete the nodes of the dag
   */
  bool tmpDag;

  /**
   * Cancelled flag (used when a node execution fails)
   */
  bool cancelled;

  /**
    * Critical section of the dag
   */
  omni_mutex myLock;

  /**
   * Recursive method for updateDelayRec
   */
  bool
  _updateDelayRec(DagNode * node, double newDelay);

  /**
   * Get port using composite id
   * (used for getting results for client API)
   */
  DagNodeOutPort *
  getOutputPort(const char* id) throw (WfStructException);
};

#endif   /* not defined _DAG_HH. */



